﻿using System.IO;
using System.Web;
using System.Web.Hosting;
using System.Web.Optimization;

namespace ChangePointSystem
{
    public class BundleConfig
    {
        // For more information on bundling, visit https://go.microsoft.com/fwlink/?LinkId=301862
        public static void RegisterBundles(BundleCollection bundles)
        {
            //Script
            bundles.Add(new ScriptBundle("~/bundles/jquery").Include(
                        "~/Scripts/jquery/jquery-{version}.js",
                        "~/Scripts/jquery/jquery-{version}.min.js",
                        "~/Scripts/jquery/jquery-{version}.intellisense.js"));

            bundles.Add(new ScriptBundle("~/bundles/jqueryval").Include(
                        "~/Scripts/validate/jquery.validate*"));

            // 開発と学習には、Modernizr の開発バージョンを使用します。次に、実稼働の準備が
            // ready for production, use the build tool at https://modernizr.com to pick only the tests you need.
            bundles.Add(new ScriptBundle("~/bundles/modernizr").Include(
                        "~/Scripts/modernizr/modernizr-*"));

            bundles.Add(new ScriptBundle("~/bundles/bootstrap").Include(
                      "~/Scripts/bootstrap.js",
                      "~/Scripts/bootstrap.min.js",
                      "~/Scripts/respond/respond.js"
                      ));

            // サジェスト
            bundles.Add(new ScriptBundle("~/bundles/chosen").Include(
                      "~/Scripts/chosen/chosen.jquery.js",
                      "~/Scripts/chosen/init.js"
                      ).WithLastModifiedToken());
            
            // カレンダー
            bundles.Add(new ScriptBundle("~/bundles/datepicker").Include(
                      "~/Scripts/bootstrap/bootstrap-datepicker.js",
                      "~/Scripts/bootstrap/bootstrap-datepicker.min.js",
                      "~/Scripts/bootstrap/bootstrap-datepicker.ja.min.js",
                      "~/Scripts/datepicker-config.js"
                      ));
            
            //originalJs
            bundles.Add(new ScriptBundle("~/bundles/menu").Include(
                      "~/Scripts/util/menu.js"                   // メニュー固定
                      , "~/Scripts/util/select-change.js"  //　SelectBox変更時、値を元に戻すjs
                      ).WithLastModifiedToken());

            //一覧用Js
            bundles.Add(new ScriptBundle("~/bundles/list").Include(
                      "~/Scripts/util/list-arrows.js"          // 一覧ソートの▲▼
                      ,"~/Scripts/util/img-approval.js"         //承認画像の表示
                      ).WithLastModifiedToken());
            
            //詳細用Js
            bundles.Add(new ScriptBundle("~/bundles/main").Include(
                      "~/Scripts/util/detail-common.js" // 詳細画面の共通
                      , "~/Scripts/util/rowspan.js"  //　RowSpan
                      , "~/Scripts/util/comma.js"
                      ).WithLastModifiedToken());

            //css

            //bootstrap
            bundles.Add(new StyleBundle("~/Content/bootstrap").Include(
                      "~/Content/bootstrap.css",
                      "~/Content/bootstrap.min.css"
                      ));

            //maincss
            bundles.Add(new StyleBundle("~/Content/css").Include(
                      "~/Content/Site.css"
                      ).WithLastModifiedToken());

            //Login
            bundles.Add(new StyleBundle("~/Content/login").Include(
                      "~/Content/Site.css",
                      "~/Content/Login.css"
                      ).WithLastModifiedToken());


            // サジェスト
            bundles.Add(new StyleBundle("~/Content/chosen").Include(
                      "~/Content/chosen/chosen.min.css"
                ));

            // サジェスト
            bundles.Add(new StyleBundle("~/Content/SpinnerBtnOffAtFireFox").Include(
                      "~/Content/SpinnerBtnOffAtFireFox.css"
                ));

            // カレンダー
            bundles.Add(new StyleBundle("~/Content/datepicker").Include(
                      "~/Content/bootstrap/bootstrap-datepicker.css",
                      "~/Content/bootstrap/bootstrap-datepicker.min.css"
                ));



        }
    }

    /// <summary>
    /// script & css にランダムな文字列を追加
    /// </summary>
    internal static class BundleExtensions
    {
        public static Bundle WithLastModifiedToken(this Bundle sb)
        {
            sb.Transforms.Add(new LastModifiedBundleTransform());
            return sb;
        }
        public class LastModifiedBundleTransform : IBundleTransform
        {
            public void Process(BundleContext context, BundleResponse response)
            {
                foreach (var file in response.Files)
                {
                    var lastWrite = File.GetLastWriteTime(HostingEnvironment.MapPath(file.IncludedVirtualPath)).Ticks.ToString();
                    file.IncludedVirtualPath = string.Concat(file.IncludedVirtualPath, "?v=", lastWrite);
                }
            }
        }
    }

}
