﻿using ChangePointSystem.Filter;
using ChangePointSystem.Util;
using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Linq;
using System.Web;
using System.Web.Mvc;

namespace ChangePointSystem.Controllers
{
    /// <summary>
    /// 全てのControllerクラス
    /// </summary>
    [LoggingFilter]
    public class AllBaseController : Controller
    {
        #region プロパティ
        protected log4net.ILog logger = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);
        #endregion

        /// <summary>
        /// エラーをログに出力
        /// </summary>
        /// <param name="filterContext"></param>
        protected override void OnException(ExceptionContext filterContext)
        {
            base.OnException(filterContext);
            logger.Error(StaticUtil.getSessionUser().UserId + ":" + filterContext.Exception.Message + ":" + filterContext.Exception.StackTrace);
        }

        /// <summary>
        /// アクション開始前
        /// </summary>
        /// <param name="filterContext"></param>
        protected override void OnActionExecuting(ActionExecutingContext filterContext)
        {
        }

        /// <summary>
        /// アクション開始後
        /// </summary>
        /// <param name="filterContext"></param>
        protected override void OnActionExecuted(ActionExecutedContext filterContext)
        {
            HttpContextBase httpContext = filterContext.HttpContext;
            bool isReturn = false;
            isReturn = httpContext.Request.Params["return"] == "true";

            //画面を戻す時だけ
            bool isRedirect
            = !(filterContext.Result is ViewResult)
                || TempData["message"] != null;
            
            //戻る用に、遷移前後のUrlを取得しておく（Get、リダイレクトは除く）
            if (httpContext.Request.RequestType == "GET"
                && !isRedirect
                )
            {

                if (System.Web.HttpContext.Current.Session["CurUrl"] != null
                    && ((Uri)System.Web.HttpContext.Current.Session["CurUrl"]).Equals(httpContext.Request.Url))
                    return;

                //戻るリストを取得
                List<Uri> returnList = new List<Uri>();
                if(System.Web.HttpContext.Current.Session["ReturnList"] != null)
                {
                    returnList.AddRange((List<Uri>)System.Web.HttpContext.Current.Session["ReturnList"]);
                }
                else
                {
                    returnList.Add(httpContext.Request.Url);
                }

                //戻るボタンが押された時、履歴を削除する
                if (isReturn)
                {

                    //戻ったので、今いるページを削除(0はじまりなので、-1する)
                    returnList.RemoveAt(returnList.Count() - 1);

                    //現ページの一つ前が戻るボタンのリンク先
                    System.Web.HttpContext.Current.Session["CurUrl"] = returnList[returnList.Count() - 2];

                }
                else
                {
                    //現ページ追加前なので、-1でOK
                    System.Web.HttpContext.Current.Session["CurUrl"] = returnList[returnList.Count() - 1];

                    //戻るじゃなければ履歴を追加
                    returnList.Add(httpContext.Request.Url);

                }
                
                System.Web.HttpContext.Current.Session["ReturnList"] = returnList;

            }

            //一つ前のページのRedirectを保持しておく
            System.Web.HttpContext.Current.Session["isRedirect"] = isRedirect;

        }
    }
}