﻿using ChangePointSystem.Models;
using ChangePointSystem.Repository;
using System.Collections.Generic;
using System;
using System.Web.Mvc;
using ChangePointSystem.Util;

namespace ChangePointSystem.Controllers

{
    [RoutePrefix("ChangeDetail")]
    public class ChangeDetailController : BaseController
    {
        [HttpGet]
        public ActionResult Index(string id, String path)
        {
            // 変化点情報取得
            ChangePointRepository changePointRepository = new ChangePointRepository();
            ChangePointViewModel changePoint = changePointRepository.FindChangePointViewById(Convert.ToInt32(id));

            // 画面表示View
            ChangeDetailViewModel viewModel = new ChangeDetailViewModel()
            {
                ChangePointId = changePoint.ChangePointId,
                LineLabel = changePoint.LineLabel,
                ProcessLabel = changePoint.ProcessLabel,
                ChangeTypeLabel = changePoint.ChangeTypeLabel
            };

            // 登録済の変化点内容を取得
            ChangeDetailRepository detailRepository = new ChangeDetailRepository();
            List<t_change_detail> detailList = detailRepository.FindChangeDetail(Convert.ToInt32(id));
            List<string> itemCodes = new List<string>();
            foreach (t_change_detail detail in detailList)
            {
                itemCodes.Add(detail.item_code);
            }
            viewModel.ItemCodes = itemCodes;
            viewModel.SelectedItem = String.Join(Const.SPLIT_COMMA_STRING, itemCodes);
            // 備考設定
            if(detailList.Count > 0)
            {
                viewModel.Comment = detailList[0].comment;
            }

            // 変化点項目リスト
            ChangeItemRepository itemRepository = new ChangeItemRepository();
            ViewBag.ItemList = itemRepository.FindChangeItemListByType(changePoint.ChangeType, changePoint.BeforeAfterType);

            // 遷移元パス設定
            if (!String.IsNullOrEmpty(path))
            {
                viewModel.SourcePath = path;

            }

            return View(viewModel);

        }
        #region 登録・更新
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Save(ChangeDetailViewModel viewModel)
        {
            int changePointId = Convert.ToInt32(viewModel.ChangePointId);
            // 変化点内容登録
            String[] itemCodes = viewModel.SelectedItem.Split(Const.SPLIT_COMMA_CHAR);
            ChangeDetailRepository detailRepository = new ChangeDetailRepository();
            t_change_detail changeDetail = null;
            if(itemCodes != null && itemCodes.Length > 0)
            {
                // 既存の変化点内容を削除
                List<t_change_detail> detailList = detailRepository.FindChangeDetail(changePointId);
                detailRepository.Delete(detailList);

                // 選択の変化点内容を登録
                for (int index = 0; index < itemCodes.Length; index++)
                {
                    changeDetail = new t_change_detail();
                    changeDetail.change_point_id = changePointId;
                    changeDetail.item_code = itemCodes[index];
                    changeDetail.priority = index + 1;
                    // 備考を最初行に登録
                    if (index == 0)
                    {
                        changeDetail.comment = viewModel.Comment;
                    }
                    detailRepository.Add(changeDetail);
                }
            }

            // 変化点情報状態登録
            ChangePointRepository repository = new ChangePointRepository();
            t_change_point changePoint = repository.FindChangePointById(changePointId);
            Boolean isUpdate = false;
            // 対応状況
            if (Const.DEAL_STATUS_CODE_NOT_REGISTE.Equals(changePoint.deal_status))
            {
                // 「10:登録済」に更新
                changePoint.deal_status = Const.DEAL_STATUS_CODE_REGISTE;
                isUpdate = true;
            }
            // 変化内容登録状態
            if (Const.REGISTE_STATUS_NOT_REGISTE.Equals(changePoint.detail_status))
            {
                // 「1:登録済」に更新
                changePoint.detail_status = Const.REGISTE_STATUS_REGISTE;
                isUpdate = true;
            }
            if(isUpdate)
            {
                repository.Update(changePoint);
            }

            // 遷移元を取得
            PathBean pathData = StaticUtil.GetReturnPath(viewModel.SourcePath, "ChangePoint");

            // 変化点情報ID
            return RedirectToAction("Index", pathData.ReturnPath, new
            {
                id = viewModel.ChangePointId,
                path = pathData.SourcePath
            });
        }
        #endregion

        #region 戻る
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Return(ChangeDetailViewModel viewModel)
        {
            // 遷移元を取得
            PathBean pathData = StaticUtil.GetReturnPath(viewModel.SourcePath, "ChangePoint");

            return RedirectToAction("Index", pathData.ReturnPath, new
            {
                id = viewModel.ChangePointId,
                path = pathData.SourcePath
            });
        }
        #endregion
    }
}
