﻿using ChangePointSystem.Models;
using ChangePointSystem.Repository;
using ChangePointSystem.Util;
using NPOI.Util;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;

namespace ChangePointSystem.Controllers

{
    [RoutePrefix("ChangeList")]
    public class ChangeListController : BaseController
    {
        [HttpGet]
        public ActionResult Index()
        {
            bool isTodayList = Convert.ToBoolean(Request.QueryString["fromToday"]);
            ChangeListViewModel viewModel = new ChangeListViewModel();

            if (isTodayList)
            {
                // 発生日Fromの初期値設定
                viewModel.AccrualDateFrom = StaticUtil.GetTomorrow();
                // 発生日Toの初期値設定
                viewModel.AccrualDateTo = null;
                //
                IList<SelectListItem> SearchTypeList = SelectListUtil.GetSearchTypeSelectList();
                SelectListItem temp = SearchTypeList[0];
                SearchTypeList[0] = SearchTypeList[1];
                SearchTypeList[1] = temp;
                ViewBag.SearchTypeList = SearchTypeList;
            }
            else {
                // 発生日Fromの初期値設定
                viewModel.AccrualDateFrom = null;
                // 発生日Toの初期値設定
                viewModel.AccrualDateTo = StaticUtil.GetToday();

                ViewBag.SearchTypeList = SelectListUtil.GetSearchTypeSelectList();


            }
            // 対応状況
            ViewBag.DealStatusList = SelectListUtil.GetStatustCheckBoxList();
            ViewBag.ChangeTypeList = SelectListUtil.GetChangeTypeSelectList();
            ViewBag.WorkTypeList = SelectListUtil.GetWorkTypeSelectList();
            ViewBag.RangeTypeList = SelectListUtil.GetRangeTypeSelectList();
            // ラインリスト
            IList<SelectListItem> LineList = SelectListUtil.GetOrganizationLineSelectList();
            ViewBag.LineList = LineList;
            return View(viewModel);
        }

        #region 変化点修正
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult ChangePointEdit(ChangeListViewModel viewModel)
        {
            // 変化点情報ID
            String changePointId = viewModel.ChangePointId;

            return RedirectToAction("Index", "ChangePoint", 
                new { id = changePointId, path = "ChangeList" });
        }
        #endregion

        #region 変化点履歴の取得
        /// <summary>
        /// ラインコードに基づく工程一覧を取得する
        /// </summary>
        /// <param name="lineCode">ラインコード</param>
        /// <returns></returns>
        [HttpPost]
        public ActionResult GetChangeList(ChangeListViewModel viewModel)
        {
            //ログインしているユーザを取得する
            string userId = StaticUtil.getSessionUser().UserId;

            // 変化点数
            string lengthString = viewModel.Length;
            int length;

            if(lengthString == "n")
            {
                length = 0;
            }
            else
            {
                length = int.Parse(lengthString);
            }

            // 変化点の発生日Fromの選択値
            Nullable<DateTime> accrualDateFrom = viewModel.AccrualDateFrom;
            // 変化点の発生日Toの選択値
            Nullable<DateTime> accrualDateTo = viewModel.AccrualDateTo;
            // ライン名の選択値
            string lineCode = viewModel.LineCode;
            if (lineCode == null)
            {
                lineCode = "";
            }
            // 4Mの選択値
            string changetype = viewModel.ChangeType;
            if (changetype == null)
            {
                changetype = "";
            }
            // 前／後工程の選択値
            string workType = viewModel.WorkType;
            if (workType == null)
            {
                workType = "";
            }
            // 工程名の選択値
            string processCode = viewModel.ProcessCode;
            if (processCode == null)
            {
                processCode = "";
            }
            // 製品・QR情報１～３の選択値
            string ProductNumber1 = viewModel.ProductNumber1;
            string ProductNumber2 = viewModel.ProductNumber2;
            string ProductNumber3 = viewModel.ProductNumber3;

            // ステータスの選択値
            string dealStatus = viewModel.DealStatus;
            // 範囲の選択値
            string rangeType = viewModel.RangeType;

            ChangePointRepository repository = new ChangePointRepository();

            IList<SelectListItem> lineList = SelectListUtil.GetOrganizationLineSelectList();
            IList<string> lineLists = lineList.Select(item => item.Value).ToList();
            //最終閲覧日時
            UserRepository userRepository = new UserRepository();
            m_user userList = userRepository.FindUser(StaticUtil.getSessionUser().UserId);
            System.DateTime? LastViewDt = userList.last_view_dt;


            List<ChangeListViewModel> changePointList = repository.FindChangeList(
                accrualDateFrom, accrualDateTo, lineCode,
                changetype, workType,  processCode,
                ProductNumber1, ProductNumber2, ProductNumber3, dealStatus, rangeType, length, lineLists);

            foreach (var item in changePointList)
            {
                //4M区分の表示ラベルを取得する
                item.ChangeType = StaticUtil.GetChangeTypeLable(item.ChangeType);
                if(item.Comment == null)
                {
                    item.Comment = "";
                }
                if (item.ItemContent == null)
                {
                    item.ItemContent = "";
                }
                DateTime tommorrow = StaticUtil.GetTomorrow();
                if (item.AccrualDate >= tommorrow) {
                    if ((item.ChangePointUser != null && !item.ChangePointUser.Equals(userId))
                        || (item.DetailUser != null && !item.DetailUser.Equals(userId))
                        || (item.TrainCheckUser != null && !item.TrainCheckUser.Equals(userId))
                        || (item.TrainingUser != null && !item.TrainingUser.Equals(userId))
                        || (item.FinalUser != null && !item.FinalUser.Equals(userId))) 
                    {
                        if (item.RegistDt > LastViewDt)
                        {
                            item.isColor = true;
                        }
                    }
                                       
                }
            }
            if (viewModel.isTodayList)
            {

                //最終閲覧日時を更新
                userList.last_view_dt = StaticUtil.GetNow();

                //更新
                userRepository.Update(userList);
            }

            return Json(changePointList, JsonRequestBehavior.AllowGet);
        }
        #endregion

        #region 工程リストの取得
        /// <summary>
        /// ラインコードに基づく工程一覧を取得する
        /// </summary>
        /// <param name="lineCode">ラインコード</param>
        /// <returns></returns>
        [HttpGet]
        public ActionResult GetProcessList(string lineCode)
        {
            // 工程一覧取得
            IList<SelectListItem> processList = SelectListUtil.GetProcessSelectList(lineCode);

            object obj = new { status = "OK", data = processList };

            return Json(obj, JsonRequestBehavior.AllowGet);
        }
        #endregion

        /// <summary>
        /// ステータス毎の画面遷移
        /// </summary>
        /// <param name="lineCode">ラインコード</param>
        /// <returns></returns>
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult NextAction(ChangeListViewModel viewModel)
        {
            // 変化点情報ID
            String changePointId = viewModel.ChangePointId;
            // 変化点情報状態登録
            ChangePointRepository repository = new ChangePointRepository();
            t_change_point changePoint = repository.FindChangePointById(Convert.ToInt32(changePointId));

            // ステータス毎の画面遷移
            String nextAction = "ChangeList";
            if (changePoint != null)
            {
                switch (changePoint.deal_status)
                {
                    case "00": //未登録
                        nextAction = "ChangeDetail";
                        break;
                    case "10": // 登録済
                        if (Const.REGISTE_STATUS_REGISTE.Equals(changePoint.training_status))
                        {
                            // 教育結果確認
                            nextAction = "TrainingResult";
                        }
                        else
                        {
                            // 変化点教育登録
                            nextAction = "Training";
                        }
                        break;
                    case "20": // 教育確認NG
                        nextAction = "TrainingResult";
                        break;
                    case "29": // 教育確認済
                        if (String.IsNullOrEmpty(changePoint.line_leader_code))
                        {
                            nextAction = "LeaderConfirm"; // ライン長登録へ
                        }
                        else
                        {
                            nextAction = "CheckResult"; // 検査結果確認 へ
                        }
                        break;
                    case "30": // L長確認待ち
                        nextAction = "LeaderConfirm"; // ライン長登録へ
                        break;
                    case "40": // 検査確認NG
                        nextAction = "CheckResult";
                        break;
                    case "49": // 検査確認済
                        nextAction = "FinalConfirm";
                        break;
                    case "50": // 最終確認NG
                        nextAction = "FinalConfirm";
                        break;
                    case "59": // 最終確認済
                        nextAction = "FinalConfirm";
                        break;
                    case "60": // 管理者承認済
                        nextAction = "FinalConfirm";
                        break;
                }
            }
            return RedirectToAction("Index", nextAction, new
            {
                id = viewModel.ChangePointId,
                path = "ChangeList"
            });
        }
    }
}