﻿using ChangePointSystem.Models;
using ChangePointSystem.Repository;
using ChangePointSystem.Util;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web.Mvc;

namespace ChangePointSystem.Controllers

{
    [RoutePrefix("ChangePoint")]
    public class ChangePointController : BaseController
    {
        [HttpGet]
        public ActionResult Index(String id, String path)
        {
            ChangePointViewModel viewModel = null;
            if (String.IsNullOrEmpty(id))
            { // 新規登録
                viewModel = new ChangePointViewModel
                {
                    // 対応状況の初期値設定
                    DealStatus = Const.DEAL_STATUS_CODE_NOT_REGISTE
                };
                viewModel.DealStatusLabel = StaticUtil.GetDealStatusLable(viewModel.DealStatus);
                // 発生日の初期値設定
                viewModel.AccrualDate = StaticUtil.GetToday();
                // 4M区分の初期値設定
                //viewModel.ChangeType = Const.CHANGE_TYPE_MAN;
                // 登録者
                viewModel.UserId = StaticUtil.getSessionUser().UserId;
                viewModel.UserName = StaticUtil.getSessionUser().UserNm;

                // ラインリスト
                IList<SelectListItem> lineList = SelectListUtil.GetOrganizationLineSelectList();
                ViewBag.LineList = lineList;
                // 工程リスト
                ViewBag.ProcessList = SelectListUtil.GetProcessSelectList(lineList.First().Value);
                // 4M区分
                ViewBag.ChangeTypeList = SelectListUtil.GetChangeTypeSelectList();
                // 前後区分
                ViewBag.BeforeAfterList = SelectListUtil.GetBeforeAfterSelectList();

                // 対応状況
                viewModel.DealStatus = Const.DEAL_STATUS_CODE_NOT_REGISTE;
                // 変化内容登録状態
                viewModel.DetailStatus = Const.REGISTE_STATUS_NOT_REGISTE;
                // 教育登録状態
                viewModel.TrainingStatus = Const.REGISTE_STATUS_NOT_REGISTE;
                // 教育結果登録状態
                viewModel.TrainingResultStatus = Const.REGISTE_STATUS_NOT_REGISTE;
                // 検査結果登録状態
                viewModel.CheckResultStatus = Const.REGISTE_STATUS_NOT_REGISTE;
                // 最終確認登録状態
                viewModel.ConfirmStatus = Const.REGISTE_STATUS_NOT_REGISTE;
            } else
            { // 更新
                // 変更点情報取得
                ChangePointRepository repository = new ChangePointRepository ();
                t_change_point changePoint = repository.FindChangePointById(Convert.ToInt32(id));
                if (changePoint != null)
                {
                    viewModel = new ChangePointViewModel(changePoint);
                }
                else
                {
                    viewModel = new ChangePointViewModel();
                    viewModel.UserId = StaticUtil.getSessionUser().UserId;
                }
                // 登録者名取得
                WorkerRepository workerRepository = new WorkerRepository();
                m_worker worker = workerRepository.FindWorker(viewModel.UserId);
                if (worker != null)
                {
                    viewModel.UserName = worker.work_name;
                }
                // ラインリスト
                IList<SelectListItem> lineList = SelectListUtil.GetOrganizationLineSelectList();
                ViewBag.LineList = lineList;
                // 工程リスト
                ViewBag.ProcessList = SelectListUtil.GetProcessSelectList(viewModel.LineCode);
                // 4M区分
                ViewBag.ChangeTypeList = SelectListUtil.GetChangeTypeSelectList();
                // 前後区分
                ViewBag.BeforeAfterList = SelectListUtil.GetBeforeAfterSelectList();
            }
            
            // 遷移元パス設定
            if(!String.IsNullOrEmpty(path))
            {
                viewModel.SourcePath = path;

            }
            return View(viewModel);
        }

        #region 登録・更新
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Save(ChangePointViewModel viewModel)
        {
            // 変化点情報ID
            String changePointId = viewModel.ChangePointId;
            ChangePointRepository repository = new ChangePointRepository();
            t_change_point entity = new t_change_point();
            if (!String.IsNullOrEmpty(changePointId))
            {
                // 更新
                entity = repository.FindChangePointById(Convert.ToInt32(changePointId));
            }
            // エフNo.
            entity.efu_no = viewModel.EfuNo;
            // 発生日
            entity.accrual_date = viewModel.AccrualDate;
            // ラインコード
            entity.line_code = viewModel.LineCode;
            // 工程コード
            entity.process_code = viewModel.ProcessCode;
            // 登録者ID
            entity.user_id = viewModel.UserId;
            // 4M区分
            entity.change_type = viewModel.ChangeType;
            // 前後区分
            entity.before_after_type = viewModel.BeforeAfterType;

            if (String.IsNullOrEmpty(changePointId))
            {
                // 新規登録
                // 対応状況
                entity.deal_status = Const.DEAL_STATUS_CODE_NOT_REGISTE;
                // 変化内容登録状態
                entity.detail_status = Const.REGISTE_STATUS_NOT_REGISTE;
                // 教育登録状態
                entity.training_status = Const.REGISTE_STATUS_NOT_REGISTE;
                // 教育結果登録状態
                entity.training_result_status = Const.REGISTE_STATUS_NOT_REGISTE;
                // 検査結果登録状態
                entity.check_result_status = Const.REGISTE_STATUS_NOT_REGISTE;
                // 最終確認登録状態
                entity.confirm_status = Const.REGISTE_STATUS_NOT_REGISTE;
                // 管理No採番
                entity.manage_no = StaticUtil.GetManageNo(viewModel.LineCode);
                repository.Add(entity);
                changePointId = Convert.ToString(entity.change_point_id);
            }
            else
            {
                // 更新処理
                repository.Update(entity);
            }

            // 変化点内容登録画面遷移判定
            if("1".Equals(viewModel.ChangeDetailFlg))
            {
                // 遷移先画面に渡す戻るパス取得
                String sourcePath = StaticUtil.GetSourcePath(viewModel.SourcePath, "ChangePoint");

                return RedirectToAction("Index", "ChangeDetail", new
                {
                    id = changePointId,
                    path = sourcePath
                });
            } else
            {
                // 遷移元を取得
                PathBean pathData = StaticUtil.GetReturnPath(viewModel.SourcePath, "TodayList");

                // 変化点情報ID
                return RedirectToAction("Index", pathData.ReturnPath, new
                {
                    id = changePointId,
                    path = pathData.SourcePath
                });
            }
        }
        #endregion

        #region 削除
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Delete(ChangePointViewModel viewModel)
        {
            // 削除変更点情報取得
            ChangePointRepository repository = new ChangePointRepository();
            t_change_point changePoint = repository.FindChangePointById(Convert.ToInt32(viewModel.ChangePointId));
            // 変更点情報削除
            repository.Delete(changePoint);

            // 遷移元を取得
            PathBean pathData = StaticUtil.GetReturnPath(viewModel.SourcePath, "TodayList");

            // 変化点情報ID
            return RedirectToAction("Index", pathData.ReturnPath, new
            {
                id = viewModel.ChangePointId,
                path = pathData.SourcePath
            });
        }
        #endregion

        #region 変化点内容登録遷移
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult ChangeDetail(ChangePointViewModel viewModel)
        {
            // 変化点情報ID
            String changePointId = viewModel.ChangePointId;

            // 遷移先画面に渡す戻るパス取得
            String sourcePath = StaticUtil.GetSourcePath(viewModel.SourcePath, "ChangePoint");

            return RedirectToAction("Index", "ChangeDetail", new { id  = changePointId,
                path = sourcePath});
        }
        #endregion

        #region 変化点教育登録遷移
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Training(ChangePointViewModel viewModel)
        {
            // 変化点情報ID
            String changePointId = viewModel.ChangePointId;
            // 遷移先画面に渡す戻るパス取得
            String sourcePath = StaticUtil.GetSourcePath(viewModel.SourcePath, "ChangePoint");

            return RedirectToAction("Index", "Training", new { id = changePointId,
                path = sourcePath});
        }
        #endregion

        #region 変化点教育結果登録遷移
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult TrainingResult(ChangePointViewModel viewModel)
        {
            // 変化点情報ID
            String changePointId = viewModel.ChangePointId;
            // 遷移先画面に渡す戻るパス取得
            String sourcePath = StaticUtil.GetSourcePath(viewModel.SourcePath, "ChangePoint");

            return RedirectToAction("Index", "TrainingResult", new { id = changePointId,
                path = sourcePath}); 
        }
        #endregion

        #region 変化点検査結果登録遷移
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult CheckResult(ChangePointViewModel viewModel)
        {
            // 変化点情報ID
            String changePointId = viewModel.ChangePointId;
            // 変化点情報取得
            ChangePointRepository repository = new ChangePointRepository();
            t_change_point changePoint = repository.FindChangePointById(Convert.ToInt32(changePointId));

            // 遷移先判定
            String nextAction = "CheckResult";
            if(Const.DEAL_STATUS_CODE_WAITE_CONFIRM.Equals(changePoint.deal_status) ||
                (Const.DEAL_STATUS_CODE_TRAINING.Equals(changePoint.deal_status) &&
                String.IsNullOrEmpty(changePoint.line_leader_code)))
            {
                nextAction = "LeaderConfirm";
            }


            // 遷移先画面に渡す戻るパス取得
            String sourcePath = StaticUtil.GetSourcePath(viewModel.SourcePath, "ChangePoint");

            return RedirectToAction("Index", nextAction, new { id = changePointId,
                path = sourcePath}); 
        }
        #endregion

        #region 変化点最終確認登録遷移
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult FinalConfirm(ChangePointViewModel viewModel)
        {
            // 変化点情報ID
            String changePointId = viewModel.ChangePointId;
            // 遷移先画面に渡す戻るパス取得
            String sourcePath = StaticUtil.GetSourcePath(viewModel.SourcePath, "ChangePoint");

            return RedirectToAction("Index", "FinalConfirm", new { id = changePointId,
                path = sourcePath});
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult FinalConfirm02(ChangePointViewModel viewModel)
        {
            // 変化点情報ID
            String changePointId = viewModel.ChangePointId;
            // 遷移先画面に渡す戻るパス取得
            String sourcePath = StaticUtil.GetSourcePath(viewModel.SourcePath, "ChangePoint");

            return RedirectToAction("Index02", "FinalConfirm", new
            {
                id = changePointId,
                path = sourcePath
            });
        }
        #endregion

        #region 戻る
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Return(ChangePointViewModel viewModel)
        {
            // 遷移元を取得
            PathBean pathData = StaticUtil.GetReturnPath(viewModel.SourcePath, "TodayList");

            // 変化点情報ID
            return RedirectToAction("Index", pathData.ReturnPath, new
            {
                id = viewModel.ChangePointId,
                path = pathData.SourcePath
            });
        }
        #endregion

        #region 工程リストの取得
        /// <summary>
        /// ラインコードに基づく工程一覧を取得する
        /// </summary>
        /// <param name="lineCode">ラインコード</param>
        /// <returns></returns>
        [HttpGet]
        public ActionResult GetProcessList(string lineCode)
        {
            // 工程一覧取得
            IList<SelectListItem> processList = SelectListUtil.GetProcessSelectList(lineCode);

            object obj = new { status = "OK", data = processList };

            return Json(obj, JsonRequestBehavior.AllowGet);
        }
        #endregion
    }
}