﻿using ChangePointSystem.Models;
using ChangePointSystem.Repository;
using ChangePointSystem.Util;
using NPOI.SS.Formula.Functions;
using OfficeOpenXml.FormulaParsing.Excel.Functions.DateTime;
using OfficeOpenXml.FormulaParsing.Excel.Functions.RefAndLookup;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web.Mvc;

namespace ChangePointSystem.Controllers

{
    [RoutePrefix("EfuList")]
    public class EfuListController : BaseController
    {
        Boolean isAllEfuListFlag;
        [HttpGet]
        public ActionResult Index(EfuListViewModel viewModelx)
        {
            isAllEfuListFlag = viewModelx.isAllEfuList;

            EfuListViewModel viewModel = new EfuListViewModel();

            //ラインの初期値設定
            IList<SelectListItem> lineList = SelectListUtil.GetOrganizationLineSelectList();
            ViewBag.lineList = lineList;

            string line;
            if (viewModelx.LineCode == null)
            {
                line = "";
            }
            else
            {
                line = viewModelx.LineCode;
            }

            //EfuListの情報を取得
            ChangePointRepository repository = new ChangePointRepository();
            IList<EfuListViewModel> EfuListViewModel = new List<EfuListViewModel>();
            if (!isAllEfuListFlag)
            {
                EfuListViewModel = repository.FindEfuList(line);
            }
            else
            {
                EfuListViewModel = repository.FindAllEfuList(line);
            }

            int EfuNoCount = 0;
            foreach (var item in EfuListViewModel)
            {
                //発生設定
                item.AccrualDateDisplay = item.AccrualDate.ToString("MM/dd");
                if(!string.IsNullOrEmpty(item.EfuNo1) && (Const.EFU_REMOVE_NOT.Equals(item.EfuRemoveConfirm1)))
                {
                    EfuNoCount++;
                }
                if (!string.IsNullOrEmpty(item.EfuNo2) && (Const.EFU_REMOVE_NOT.Equals(item.EfuRemoveConfirm2)))
                {
                    EfuNoCount++;
                }
                if (!string.IsNullOrEmpty(item.EfuNo3) && (Const.EFU_REMOVE_NOT.Equals(item.EfuRemoveConfirm3)))
                {
                    EfuNoCount++;
                }
            }
            viewModel.EfuNoCount = EfuNoCount;
            viewModel.isAllEfuList = isAllEfuListFlag;
            ViewData["list"] = EfuListViewModel;

            return View(viewModel);

        }

        #region Efuリストの取得
        /// <summary>
        /// ラインコードに基づくEfuList一覧を取得する
        /// </summary>
        /// <param name="lineCode">ラインコード</param>
        /// <returns></returns>
        [HttpGet]
        public ActionResult GetEfuListByLine(string LineCode, Boolean isAllEfuList)
        {
            isAllEfuListFlag = isAllEfuList;

            EfuListViewModel viewModel = new EfuListViewModel();

            //EfuListの情報を取得
            ChangePointRepository repository = new ChangePointRepository();

            IList<EfuListViewModel> EfuListByLine = new List<EfuListViewModel>();

            if (!isAllEfuListFlag)
            {
                EfuListByLine = repository.FindEfuList(LineCode);
            }
            else
            {
                EfuListByLine = repository.FindAllEfuList(LineCode);
            }

            int EfuNoCount = 0;

            foreach (var item in EfuListByLine)
            {
                //発生設定
                item.AccrualDateDisplay = item.AccrualDate.ToString("MM/dd");
                var type = item.GetType();
                var properties = type.GetProperties();
                foreach (var property in properties)
                {
                    var propertyValue = property.GetValue(item);

                    if (propertyValue == null)
                    {
                        property.SetValue(item, "");
                    }
                }

                if (!string.IsNullOrEmpty(item.EfuNo1) && (Const.EFU_REMOVE_NOT.Equals(item.EfuRemoveConfirm1)))
                {
                    EfuNoCount++;
                }
                if (!string.IsNullOrEmpty(item.EfuNo2) && (Const.EFU_REMOVE_NOT.Equals(item.EfuRemoveConfirm2)))
                {
                    EfuNoCount++;
                }
                if (!string.IsNullOrEmpty(item.EfuNo3) && (Const.EFU_REMOVE_NOT.Equals(item.EfuRemoveConfirm3)))
                {
                    EfuNoCount++;
                }
            }

            
            ViewData["list"] = EfuListByLine;

            object obj = new { status = "OK", data = EfuListByLine ,count = EfuNoCount };

            return Json(obj, JsonRequestBehavior.AllowGet);
        }
        #endregion

        #region 全部のEfuリストの取得
        /// <summary>
        /// ラインコードに基づく全部のEfuList一覧を取得する
        /// </summary>
        /// <param name="lineCode">ラインコード</param>
        /// <returns></returns>
        [HttpGet]
        public ActionResult GetAllEfuListByLine(string LineCode)
        {
            EfuListViewModel viewModel = new EfuListViewModel();

            //EfuListの情報を取得
            ChangePointRepository repository = new ChangePointRepository();
            IList<EfuListViewModel> AllEfuListByLine = repository.FindAllEfuList(LineCode);

            ViewData["list"] = AllEfuListByLine;

            int EfuNoCount = 0;

            foreach (var item in AllEfuListByLine)
            {
                //発生設定
                item.AccrualDateDisplay = item.AccrualDate.ToString("MM/dd");
                var type = item.GetType();
                var properties = type.GetProperties();
                foreach (var property in properties)
                {
                    var propertyValue = property.GetValue(item);

                    if (propertyValue == null)
                    {
                        property.SetValue(item, "");
                    }
                }
                if (!string.IsNullOrEmpty(item.EfuNo1) && (Const.EFU_REMOVE_NOT.Equals(item.EfuRemoveConfirm1)))
                {
                    EfuNoCount++;
                }
                if (!string.IsNullOrEmpty(item.EfuNo2) && (Const.EFU_REMOVE_NOT.Equals(item.EfuRemoveConfirm2)))
                {
                    EfuNoCount++;
                }
                if (!string.IsNullOrEmpty(item.EfuNo3) && (Const.EFU_REMOVE_NOT.Equals(item.EfuRemoveConfirm3)))
                {
                    EfuNoCount++;
                }
            }

            object obj = new { status = "OK", data = AllEfuListByLine, count = EfuNoCount };

            return Json(obj, JsonRequestBehavior.AllowGet);
        }
        #endregion

        #region 反映
        /// <summary>
        /// 
        /// </summary>
        /// <param name="viewModel">ビューデータ</param>
        /// <param name="isAllEfuList">スキャン済表示フラグ</param>
        /// <returns></returns>
        [HttpPost]
        public ActionResult Save(EfuListViewModel viewModel, bool isAllEfuList)
        {
            viewModel.isAllEfuList = isAllEfuList;

            //ラインの初期値設定
            IList<SelectListItem> lineList = SelectListUtil.GetOrganizationLineSelectList();
            ViewBag.lineList = lineList;

            // 検査確認情報取得
            TrainingCheckResultRepository repository = new TrainingCheckResultRepository();
            t_training_check_result checkResult = repository.FindCheckResultByEfuNo1(viewModel.LineCode, viewModel.EfuList);

            if (checkResult == null)
            {
                // エフNo登録していません
                ViewBag.ErrorMessage = String.Format(Resources.message.E525, viewModel.EfuList);
                // 一覧データ取得
                GetEfuList(viewModel);
                return View("Index", viewModel);
            }

            // エフNo1チェック
            if (checkResult.efu_no1.Equals(viewModel.EfuList))
            {
                // 取り外し登録済チェック
                if (checkResult.efu_remove_confirm1 != Const.EFU_REMOVE_OK)
                {
                    checkResult.efu_remove_confirm1 = Const.EFU_REMOVE_OK;
                }
            }
            // エフNo2チェック
            else if (checkResult.efu_no2 == viewModel.EfuList)
            {
                // 取り外し登録済チェック
                if (checkResult.efu_remove_confirm2 != Const.EFU_REMOVE_OK)
                {
                    checkResult.efu_remove_confirm2 = Const.EFU_REMOVE_OK;
                }
            }
            // エフNo3チェック
            else if (checkResult.efu_no3 == viewModel.EfuList)
            {
                // 取り外し登録済チェック
                if (checkResult.efu_remove_confirm3 != Const.EFU_REMOVE_OK)
                {
                    checkResult.efu_remove_confirm3 = Const.EFU_REMOVE_OK;
                }
            }

            // エフNo取り外し登録
            repository.efuNo1Reflection(checkResult);

            // 一覧データ取得
            GetEfuList(viewModel);

            return View("Index", viewModel);
        }
        #endregion

        /// <summary>
        /// 
        /// </summary>
        /// <param name="viewModel">ビューデータ</param>
        /// <returns></returns>
        private void GetEfuList(EfuListViewModel viewModel)
        {
            IList<EfuListViewModel> EfuList = new List<EfuListViewModel>();
            ChangePointRepository repository2 = new ChangePointRepository();
            if (viewModel.isAllEfuList)
            {
                EfuList = repository2.FindAllEfuList(viewModel.LineCode);
            }
            else
            {
                EfuList = repository2.FindEfuList(viewModel.LineCode);

            }

            int EfuNoCount = 0;
            foreach (var item in EfuList)
            {
                //発生設定
                item.AccrualDateDisplay = item.AccrualDate.ToString("MM/dd");
                if (!string.IsNullOrEmpty(item.EfuNo1) && (Const.EFU_REMOVE_NOT.Equals(item.EfuRemoveConfirm1)))
                {
                    EfuNoCount++;
                }
                if (!string.IsNullOrEmpty(item.EfuNo2) && (Const.EFU_REMOVE_NOT.Equals(item.EfuRemoveConfirm2)))
                {
                    EfuNoCount++;
                }
                if (!string.IsNullOrEmpty(item.EfuNo3) && (Const.EFU_REMOVE_NOT.Equals(item.EfuRemoveConfirm3)))
                {
                    EfuNoCount++;
                }
            }
            viewModel.EfuNoCount = EfuNoCount;
            ViewData["list"] = EfuList;
        }
    }
}
    