﻿using ChangePointSystem.Models;
using ChangePointSystem.Repository;
using ChangePointSystem.Util;
using System;
using System.Collections.Generic;
using System.Web.Mvc;

namespace ChangePointSystem.Controllers

{
    [RoutePrefix("EfuRemove")]
    public class EfuRemoveController : BaseController
    {
        [HttpGet]
        public ActionResult Index(string id, String path)
        {
            // 変化点情報取得
            ChangePointRepository changePointRepository = new ChangePointRepository();
            ChangePointViewModel changePoint = changePointRepository.FindChangePointViewById(Convert.ToInt32(id));

            // 画面表示View
            EfuRemoveViewModel viewModel = new EfuRemoveViewModel();
            viewModel.ChangePointId = changePoint.ChangePointId;
            viewModel.ChangeTypeLabel = changePoint.ChangeTypeLabel;
            viewModel.LineLabel = changePoint.LineLabel;
            viewModel.ProcessLabel = changePoint.ProcessLabel;

            //検査確認情報取得
            int changePointIds = Convert.ToInt32(id);
            TrainingCheckResultRepository CheckRepository = new TrainingCheckResultRepository();
            CheckResultViewModel CheckViewModel = CheckRepository.FindCheckResultById(changePointIds);
            viewModel.EfuNoCount = 0;
            if (!string.IsNullOrEmpty(CheckViewModel.EfuNo1)) {
                viewModel.EfuNoCount++;
            }
            if (!string.IsNullOrEmpty(CheckViewModel.EfuNo2))
            {
                viewModel.EfuNoCount++;
            }
            if (!string.IsNullOrEmpty(CheckViewModel.EfuNo3))
            {
                viewModel.EfuNoCount++;
            }

            // 遷移元パス設定
            if (!String.IsNullOrEmpty(path))
            {
                viewModel.SourcePath = path;

            }

            return View(viewModel);
        }

        #region 次へ
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Next(EfuRemoveViewModel viewModel)
        {
            
            int changePointId = Convert.ToInt32(viewModel.ChangePointId);
            // 検査確認情報取得
            TrainingCheckResultRepository repository = new TrainingCheckResultRepository();
            t_training_check_result checkResult = repository.FindCheckResult(changePointId);

            // エフNo取り外し状況取得
            Dictionary<String, String> removeResultDic = new Dictionary<string, string>();
            if(checkResult != null)
            {
                // エフNo1取り外し状況
                if (!String.IsNullOrEmpty(checkResult.efu_no1))
                {
                    removeResultDic.Add(checkResult.efu_no1, checkResult.efu_remove_confirm1);
                }
                // エフNo2取り外し状況
                if (!String.IsNullOrEmpty(checkResult.efu_no2))
                {
                    removeResultDic.Add(checkResult.efu_no2, checkResult.efu_remove_confirm2);
                }
                // エフNo3取り外し状況
                if (!String.IsNullOrEmpty(checkResult.efu_no3))
                {
                    removeResultDic.Add(checkResult.efu_no3, checkResult.efu_remove_confirm3);
                }
            }

            // エフNo1チェック
            if (!String.IsNullOrEmpty(viewModel.EfuNo1))
            {
                if(removeResultDic.ContainsKey(viewModel.EfuNo1))
                {
                    if(Const.EFU_REMOVE_OK.Equals(removeResultDic[viewModel.EfuNo1]))
                    {
                        // 取り外し登録済エラー
                        ViewBag.ErrorMessage = Resources.message.E513;
                        return View("Index", viewModel);
                    }
                    // エフNo取り外し登録
                    if(viewModel.EfuNo1.Equals(checkResult.efu_no1))
                    {
                        checkResult.efu_remove_confirm1 = Const.EFU_REMOVE_OK;
                    } 
                    else if(viewModel.EfuNo1.Equals(checkResult.efu_no2))
                    {
                        checkResult.efu_remove_confirm2 = Const.EFU_REMOVE_OK;
                    }
                    else if (viewModel.EfuNo1.Equals(checkResult.efu_no3))
                    {
                        checkResult.efu_remove_confirm3 = Const.EFU_REMOVE_OK;
                    }
                }
                else
                {
                    // 未登録エフNoエラー
                    ViewBag.ErrorMessage = String.Format(Resources.message.E525, viewModel.EfuNo1);
                    return View("Index", viewModel);
                }
                
            }

            // エフNo2チェック
            if (!String.IsNullOrEmpty(viewModel.EfuNo2))
            {
                if (removeResultDic.ContainsKey(viewModel.EfuNo2))
                {
                    if (Const.EFU_REMOVE_OK.Equals(removeResultDic[viewModel.EfuNo2]))
                    {
                        // 取り外し登録済エラー
                        ViewBag.ErrorMessage = Resources.message.E513;
                        return View("Index", viewModel);
                    }
                    // エフNo取り外し登録
                    if (viewModel.EfuNo2.Equals(checkResult.efu_no1))
                    {
                        checkResult.efu_remove_confirm1 = Const.EFU_REMOVE_OK;
                    }
                    else if (viewModel.EfuNo2.Equals(checkResult.efu_no2))
                    {
                        checkResult.efu_remove_confirm2 = Const.EFU_REMOVE_OK;
                    }
                    else if (viewModel.EfuNo2.Equals(checkResult.efu_no3))
                    {
                        checkResult.efu_remove_confirm3 = Const.EFU_REMOVE_OK;
                    }
                }
                else
                {
                    // 未登録エフNoエラー
                    ViewBag.ErrorMessage = String.Format(Resources.message.E525, viewModel.EfuNo2);
                    return View("Index", viewModel);
                }

            }

            // エフNo3チェック
            if (!String.IsNullOrEmpty(viewModel.EfuNo3))
            {
                if (removeResultDic.ContainsKey(viewModel.EfuNo3))
                {
                    if (Const.EFU_REMOVE_OK.Equals(removeResultDic[viewModel.EfuNo3]))
                    {
                        // 取り外し登録済エラー
                        ViewBag.ErrorMessage = Resources.message.E513;
                        return View("Index", viewModel);
                    }
                    // エフNo取り外し登録
                    if (viewModel.EfuNo3.Equals(checkResult.efu_no1))
                    {
                        checkResult.efu_remove_confirm1 = Const.EFU_REMOVE_OK;
                    }
                    else if (viewModel.EfuNo3.Equals(checkResult.efu_no2))
                    {
                        checkResult.efu_remove_confirm2 = Const.EFU_REMOVE_OK;
                    }
                    else if (viewModel.EfuNo3.Equals(checkResult.efu_no3))
                    {
                        checkResult.efu_remove_confirm3 = Const.EFU_REMOVE_OK;
                    }
                }
                else
                {
                    // 未登録エフNoエラー
                    ViewBag.ErrorMessage = String.Format(Resources.message.E525, viewModel.EfuNo3);
                    return View("Index", viewModel);
                }

            }

            // エフNo取り外し登録
            repository.Update(checkResult);

            // エフ取外し確認全て完了判定
            //if ((!String.IsNullOrEmpty(checkResult.efu_no1) && checkResult.efu_remove_confirm1 != Const.EFU_REMOVE_OK) ||
            //    (!String.IsNullOrEmpty(checkResult.efu_no2) && checkResult.efu_remove_confirm2 != Const.EFU_REMOVE_OK) ||
            //    (!String.IsNullOrEmpty(checkResult.efu_no3) && checkResult.efu_remove_confirm3 != Const.EFU_REMOVE_OK))
            //{
            //    if (ModelState.IsValid)
            //    {
            //        ModelState.Clear();
            //    }   
            //    viewModel.EfuNo1 = "";
            //    viewModel.EfuNo2 = "";
            //    viewModel.EfuNo3 = "";
            //    return View("Index", viewModel);
            //}
            //else
            //{
                // 遷移元を取得
                PathBean pathData = StaticUtil.GetReturnPath(viewModel.SourcePath, "ChangePoint");

                return RedirectToAction("Index", pathData.ReturnPath, new
                {
                    id = viewModel.ChangePointId,
                    path = pathData.SourcePath
                });
            //}
        }
        #endregion

        #region あとで確認
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Skip(EfuRemoveViewModel viewModel)
        {

            // 遷移元を取得
            PathBean pathData = StaticUtil.GetReturnPath(viewModel.SourcePath, "ChangePoint");

            return RedirectToAction("Index", pathData.ReturnPath, new
            {
                id = viewModel.ChangePointId,
                path = pathData.SourcePath
            });
        }
        #endregion

        #region 戻る
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Return(EfuRemoveViewModel viewModel)
        {
            // 検査結果確認へ遷移
            return RedirectToAction("Index", "CheckResult", new
            {
                id = viewModel.ChangePointId,
                path = viewModel.SourcePath
            });
        }
        #endregion
    }
}