﻿using ChangePointSystem.Models;
using ChangePointSystem.Repository;
using ChangePointSystem.Util;
using System;
using System.Web.Mvc;

namespace ChangePointSystem.Controllers

{
    [RoutePrefix("FinalConfirm")]
    public class FinalConfirmController : BaseController
    {
        [HttpGet]
        public ActionResult Index(string id, String path)
        {
            // 最終確認情報取得
            FinalConfirmRepository finalConfirmRepository = new FinalConfirmRepository();
            t_final_confirm finalConfirm = finalConfirmRepository.FindFinalConfirmById(Convert.ToInt32(id));

            // 変化点情報取得
            ChangePointRepository changePointRepository = new ChangePointRepository();
            ChangePointViewModel changePoint = changePointRepository.FindChangePointViewById(Convert.ToInt32(id));

            // 画面表示View
            FinalConfirmViewModel viewModel;
            if(finalConfirm != null)
            {
                viewModel = new FinalConfirmViewModel(finalConfirm);
            } else
            {
                viewModel = new FinalConfirmViewModel();
            }
            viewModel.ChangePointId = changePoint.ChangePointId;
            viewModel.LineLabel = changePoint.LineLabel;
            viewModel.ProcessLabel = changePoint.ProcessLabel;
            viewModel.ChangeTypeLabel = changePoint.ChangeTypeLabel;

            //担当者WebGridの取得
            ViewBag.gridD = StaticUtil.GetUserGrid(null);
            ViewBag.gridColumnD = StaticUtil.GetUserGridColumn(ViewBag.gridD);

            // 確認結果リスト
            ViewBag.ConfirmResultList = SelectListUtil.GetConfirmResultSelectList();

            // 遷移元パス設定
            if (!String.IsNullOrEmpty(path))
            {
                viewModel.SourcePath = path;

            }

            return View(viewModel);
        }

        [HttpGet]
        public ActionResult Index02(string id, String path)
        {
            // 最終確認情報取得
            FinalConfirmRepository finalConfirmRepository = new FinalConfirmRepository();
            t_final_confirm finalConfirm = finalConfirmRepository.FindFinalConfirmById(Convert.ToInt32(id));

            // 変化点情報取得
            ChangePointRepository changePointRepository = new ChangePointRepository();
            ChangePointViewModel changePoint = changePointRepository.FindChangePointViewById(Convert.ToInt32(id));

            // 画面表示View
            FinalConfirmViewModel viewModel;
            if (finalConfirm != null)
            {
                viewModel = new FinalConfirmViewModel(finalConfirm);
            }
            else
            {
                viewModel = new FinalConfirmViewModel();
            }
            viewModel.ChangePointId = changePoint.ChangePointId;
            viewModel.LineLabel = changePoint.LineLabel;
            viewModel.ProcessLabel = changePoint.ProcessLabel;
            viewModel.ChangeTypeLabel = changePoint.ChangeTypeLabel;

            //担当者WebGridの取得
            ViewBag.gridD = StaticUtil.GetUserGrid(null);
            ViewBag.gridColumnD = StaticUtil.GetUserGridColumn(ViewBag.gridD);

            // 確認結果リスト
            ViewBag.ConfirmResultList = SelectListUtil.GetConfirmResultSelectList();

            // 遷移元パス設定
            if (!String.IsNullOrEmpty(path))
            {
                viewModel.SourcePath = path;

            }

            return View(viewModel);
        }

        #region 登録・更新
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Save(FinalConfirmViewModel viewModel)
        {
            int changePointId = Convert.ToInt32(viewModel.ChangePointId);
            // 最終確認登録
            FinalConfirmRepository finalConfirmRepository = new FinalConfirmRepository();
            // 既存の最終確認情報取得
            t_final_confirm finalConfirm = finalConfirmRepository.FindFinalConfirmById(Convert.ToInt32(changePointId));

            if (finalConfirm == null)
            {
                // 最終確認情報新規登録
                finalConfirm = new t_final_confirm()
                {
                    change_point_id = changePointId,
                    result1 = viewModel.Result1,
                    result2 = viewModel.Result2,
                    confirm_user_id = viewModel.ConfirmUserId,
                    confirm_user_name = viewModel.ConfirmUserName,
                    ng_detail = viewModel.NgDetail,
                    deal_detail = viewModel.DealDetail
                };
                finalConfirmRepository.Add(finalConfirm);
            } else
            {
                // 最終確認情報更新
                finalConfirm.change_point_id = changePointId;
                finalConfirm.result1 = viewModel.Result1;
                finalConfirm.result2 = viewModel.Result2;
                finalConfirm.confirm_user_id = viewModel.ConfirmUserId;
                finalConfirm.confirm_user_name = viewModel.ConfirmUserName;
                finalConfirm.ng_detail = viewModel.NgDetail;
                finalConfirm.deal_detail = viewModel.DealDetail;
                finalConfirmRepository.Update(finalConfirm);
            }

            // 変化点情報状態登録
            ChangePointRepository repository = new ChangePointRepository();
            t_change_point changePoint = repository.FindChangePointById(changePointId);
            Boolean isUpdate = false;
            // 対応状況
            String dealStatus = changePoint.deal_status;
            if(Const.CONFIRM_RESULT_OK.Equals(viewModel.Result1) &&
                Const.CONFIRM_RESULT_OK.Equals(viewModel.Result2) &&
                !String.IsNullOrEmpty(viewModel.ConfirmUserName))
            {
                dealStatus = Const.DEAL_STATUS_CODE_FINAL_CONFIRM;
            }

            if((Const.CONFIRM_RESULT_NG.Equals(viewModel.Result1) ||
                Const.CONFIRM_RESULT_NG.Equals(viewModel.Result2))  && 
                !String.IsNullOrEmpty(viewModel.ConfirmUserName))
            {
                dealStatus = Const.DEAL_STATUS_CODE_FINAL_CONFIRM_NG;
            }

            if ((Const.DEAL_STATUS_CODE_CHECK.Equals(changePoint.deal_status) ||
                Const.DEAL_STATUS_CODE_FINAL_CONFIRM.Equals(changePoint.deal_status) ||
                Const.DEAL_STATUS_CODE_FINAL_CONFIRM_NG.Equals(changePoint.deal_status)) &&
                !dealStatus.Equals(changePoint.deal_status))
            {
                // 最終確認結果更新
                changePoint.deal_status = dealStatus;
                isUpdate = true;
            }

            // 最終確認登録状態
            if (Const.REGISTE_STATUS_NOT_REGISTE.Equals(changePoint.confirm_status))
            {
                // 「1:登録済」に更新
                changePoint.confirm_status = Const.REGISTE_STATUS_REGISTE;
                isUpdate = true;
            }

            if ((changePoint.deal_status == Const.DEAL_STATUS_CODE_FINAL_CONFIRM || changePoint.deal_status == Const.DEAL_STATUS_CODE_FINAL_CONFIRM_NG)
                && String.IsNullOrEmpty(viewModel.ConfirmUserName)) {
                changePoint.deal_status = Const.DEAL_STATUS_CODE_CHECK;
                isUpdate = true;
            }
            if (isUpdate)
            {
                repository.Update(changePoint);
            }

            // 遷移元を取得
            PathBean pathData = StaticUtil.GetReturnPath(viewModel.SourcePath, "ChangePoint");

            // 変化点情報ID
            return RedirectToAction("Index", pathData.ReturnPath, new
            {
                id = viewModel.ChangePointId,
                path = pathData.SourcePath
            });
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Save02(FinalConfirmViewModel viewModel)
        {
            int changePointId = Convert.ToInt32(viewModel.ChangePointId);
            // 最終確認登録
            FinalConfirmRepository finalConfirmRepository = new FinalConfirmRepository();
            // 既存の最終確認情報取得
            t_final_confirm finalConfirm = finalConfirmRepository.FindFinalConfirmById(Convert.ToInt32(changePointId));

            if (finalConfirm == null)
            {
                // 最終確認情報新規登録
                finalConfirm = new t_final_confirm()
                {
                    change_point_id = changePointId,
                    result1 = viewModel.Result1,
                    result2 = viewModel.Result2,
                    confirm_user_id = viewModel.ConfirmUserId,
                    confirm_user_name = viewModel.ConfirmUserName,
                    ng_detail = viewModel.NgDetail,
                    deal_detail = viewModel.DealDetail,
                };
                finalConfirmRepository.Add(finalConfirm);
            }
            else
            {
                // 最終確認情報更新
                finalConfirm.change_point_id = changePointId;
                finalConfirm.result1 = viewModel.Result1;
                finalConfirm.result2 = viewModel.Result2;
                finalConfirm.confirm_user_id = viewModel.ConfirmUserId;
                finalConfirm.confirm_user_name = viewModel.ConfirmUserName;
                finalConfirm.ng_detail = viewModel.NgDetail;
                finalConfirm.deal_detail = viewModel.DealDetail;
                finalConfirmRepository.Update(finalConfirm);
            }

            // 変化点情報状態登録
            ChangePointRepository repository = new ChangePointRepository();
            t_change_point changePoint = repository.FindChangePointById(changePointId);
            Boolean isUpdate = true;
            // 対応状況
            String dealStatus = changePoint.deal_status;
            if (Const.CONFIRM_RESULT_OK.Equals(viewModel.Result1) &&
                Const.CONFIRM_RESULT_OK.Equals(viewModel.Result2) &&
                !String.IsNullOrEmpty(viewModel.ConfirmUserName))
            {
                dealStatus = Const.DEAL_STATUS_CODE_FINAL_CONFIRM;
            }

            if ((Const.CONFIRM_RESULT_NG.Equals(viewModel.Result1) ||
                Const.CONFIRM_RESULT_NG.Equals(viewModel.Result2)) &&
                !String.IsNullOrEmpty(viewModel.ConfirmUserName))
            {
                dealStatus = Const.DEAL_STATUS_CODE_FINAL_CONFIRM_NG;
            }

            //if ((Const.DEAL_STATUS_CODE_CHECK.Equals(changePoint.deal_status) ||
            //    Const.DEAL_STATUS_CODE_FINAL_CONFIRM.Equals(changePoint.deal_status) ||
            //    Const.DEAL_STATUS_CODE_FINAL_CONFIRM_NG.Equals(changePoint.deal_status)) &&
            //    !dealStatus.Equals(changePoint.deal_status))
            //{
            //    // 最終確認結果更新
            //    changePoint.deal_status = dealStatus;
            //    isUpdate = true;
            //}

            //// 最終確認登録状態
            //if (Const.REGISTE_STATUS_NOT_REGISTE.Equals(changePoint.confirm_status))
            //{
            //    // 「1:登録済」に更新
            //    changePoint.confirm_status = Const.REGISTE_STATUS_REGISTE;
            //    isUpdate = true;
            //}

            //if ((changePoint.deal_status == Const.DEAL_STATUS_CODE_FINAL_CONFIRM || changePoint.deal_status == Const.DEAL_STATUS_CODE_FINAL_CONFIRM_NG)
            //    && String.IsNullOrEmpty(viewModel.ConfirmUserName))
            //{
            //    changePoint.deal_status = Const.DEAL_STATUS_CODE_CHECK;
            //    isUpdate = true;
            //}
            if (isUpdate)
            {
                repository.Update(changePoint);
            }

            // 遷移元を取得
            PathBean pathData = StaticUtil.GetReturnPath(viewModel.SourcePath, "ChangePoint");

            // 変化点情報ID
            return RedirectToAction("Index", pathData.ReturnPath, new
            {
                id = viewModel.ChangePointId,
                path = pathData.SourcePath
            });
        }
        #endregion

        #region 戻る
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Return(FinalConfirmViewModel viewModel)
        {
            // 遷移元を取得
            PathBean pathData = StaticUtil.GetReturnPath(viewModel.SourcePath, "ChangePoint");

            return RedirectToAction("Index", pathData.ReturnPath, new
            {
                id = viewModel.ChangePointId,
                path = pathData.SourcePath
            });
        }
        #endregion
    }
}