﻿using ChangePointSystem.Models;
using ChangePointSystem.Repository;
using ChangePointSystem.Util;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Web.WebPages;

namespace ChangePointSystem.Controllers

{
    [RoutePrefix("LeaderConfirm")]
    public class LeaderConfirmController : BaseController
    {
        [HttpGet]
        public ActionResult Index(string id, String path)
        {
            // 変化点情報取得
            ChangePointRepository changePointRepository = new ChangePointRepository();
            ChangePointViewModel changePoint = changePointRepository.FindChangePointViewById(Convert.ToInt32(id));

            // 画面表示View
            LeaderConfirmViewModel viewModel = new LeaderConfirmViewModel();
            viewModel.ChangePointId = changePoint.ChangePointId;
            viewModel.ChangeTypeLabel = changePoint.ChangeTypeLabel;
            viewModel.LineLabel = changePoint.LineLabel;
            viewModel.ProcessLabel = changePoint.ProcessLabel;
            
            // 遷移元パス設定
            if (!String.IsNullOrEmpty(path))
            {
                viewModel.SourcePath = path;

            }

            return View(viewModel);
        }

        #region 次へ
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Next(LeaderConfirmViewModel viewModel)
        {
            int changePointId = Convert.ToInt32(viewModel.ChangePointId);
            // ライン長コードチェック
            UserRepository userRepository = new UserRepository();
            m_user user = userRepository.FindUser(viewModel.LeaderId);
            if(user == null)
            {
                // ライン長コード存在しない
                ViewBag.ErrorMessage = Resources.message.E510;
                return View("Index", viewModel);
            }
            if(user.position != Const.POSITION_LINE_LEADER)
            {
                // ライン長ではない
                ViewBag.ErrorMessage = Resources.message.E511;
                return View("Index", viewModel);
            }
            // 変化点情報更新
            ChangePointRepository repository = new ChangePointRepository();
            t_change_point changePoint = repository.FindChangePointById(changePointId);
            Boolean isUpdate = false;

            // 対応状況
            if (Const.DEAL_STATUS_CODE_WAITE_CONFIRM.Equals(changePoint.deal_status))
            {
                //検査確認データを取得
                TrainingCheckResultRepository CheckResultRepository = new TrainingCheckResultRepository();
                t_training_check_result CheckREsult = CheckResultRepository.FindCheckResult(changePointId);

                if (CheckREsult == null)
                {
                    // 教育確認済に更新
                    changePoint.deal_status = Const.DEAL_STATUS_CODE_TRAINING;
                    isUpdate = true;
                }
                else
                {
                    //CheckResultの状況
                    string[][] ReconfirmResult = new string[3][];
                    string[] ReconfirmResult_status = new string[3];
                    ReconfirmResult[0] = new string[7] { CheckREsult.efu_no1, CheckREsult.result1, CheckREsult.reconfirm_result1_1, CheckREsult.reconfirm_result1_2, CheckREsult.reconfirm_result1_3, CheckREsult.reconfirm_result1_4, CheckREsult.reconfirm_result1_5 };
                    ReconfirmResult[1] = new string[7] { CheckREsult.efu_no2, CheckREsult.result2, CheckREsult.reconfirm_result2_1, CheckREsult.reconfirm_result2_2, CheckREsult.reconfirm_result2_3, CheckREsult.reconfirm_result2_4, CheckREsult.reconfirm_result2_5 };
                    ReconfirmResult[2] = new string[7] { CheckREsult.efu_no3, CheckREsult.result3, CheckREsult.reconfirm_result3_1, CheckREsult.reconfirm_result3_2, CheckREsult.reconfirm_result3_3, CheckREsult.reconfirm_result3_4, CheckREsult.reconfirm_result3_5 };

                    //CheckResult OK または NGの判断　
                    ReconfirmResult_status = StaticUtil.getReconfirmResult(ReconfirmResult);

                    //CheckResultはNGの場合
                    if (ReconfirmResult_status.Contains("NG"))
                    {
                        changePoint.deal_status = Const.DEAL_STATUS_CODE_CHECK_NG;
                        isUpdate = true;
                    }
                    //CheckResultはOKの場合
                    else if (ReconfirmResult_status.Contains("OK"))
                    {
                        //「30:検査確認済」に更新
                        changePoint.deal_status = Const.DEAL_STATUS_CODE_CHECK;
                        isUpdate = true;
                    }
                    else
                    {
                        // 教育確認済に更新
                        changePoint.deal_status = Const.DEAL_STATUS_CODE_TRAINING;
                        isUpdate = true;
                    }
                }
            }

            // ライン長コード更新
            if (!String.IsNullOrEmpty(viewModel.LeaderId))
            {
                // ライン長コード設定
                changePoint.line_leader_code = viewModel.LeaderId;
                isUpdate = true;
            }
            if (isUpdate)
            {
                repository.Update(changePoint);
            }

            // 検査結果確認画面へ遷移
            return RedirectToAction("Index", "CheckResult", new
            {
                id = viewModel.ChangePointId,
                path = viewModel.SourcePath
            });
        }
        #endregion

        #region あとで確認
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Skip(LeaderConfirmViewModel viewModel)
        {
            int changePointId = Convert.ToInt32(viewModel.ChangePointId);

            // 変化点情報更新
            ChangePointRepository repository = new ChangePointRepository();
            t_change_point changePoint = repository.FindChangePointById(changePointId);
            Boolean isUpdate = false;
            // 対応状況
            if (Const.DEAL_STATUS_CODE_TRAINING.Equals(changePoint.deal_status))
            {
                // L長確認待ちに更新
                changePoint.deal_status = Const.DEAL_STATUS_CODE_WAITE_CONFIRM;
                isUpdate = true;
            }

            if (isUpdate)
            {
                repository.Update(changePoint);
            }

            // 検査結果確認画面へ遷移
            return RedirectToAction("Index", "CheckResult", new
            {
                id = viewModel.ChangePointId,
                path = viewModel.SourcePath
            });
        }
        #endregion

        #region 戻る
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Return(LeaderConfirmViewModel viewModel)
        {
            // 遷移元を取得
            PathBean pathData = StaticUtil.GetReturnPath(viewModel.SourcePath, "ChangePoint");

            return RedirectToAction("Index", pathData.ReturnPath, new
            {
                id = viewModel.ChangePointId,
                path = pathData.SourcePath
            });
        }
        #endregion
    }
}