﻿using ChangePointSystem.Models;
using ChangePointSystem.Repository;
using ChangePointSystem.Util;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web.Mvc;

namespace ChangePointSystem.Controllers

{
    [RoutePrefix("TodayList")]
    public class TodayListController : BaseController
    {
        [HttpGet]
        public ActionResult Index()
        {
            TodayListViewModel viewModel = new TodayListViewModel();

            //ラインの初期値設定
            IList<SelectListItem> lineList = SelectListUtil.GetOrganizationLineSelectList();
            ViewBag.lineList = lineList;

            // ライン条件取得
            String lineCode = lineList.First().Value;
            if (System.Web.HttpContext.Current.Session["TodayList_Condition"] != null)
            {
                lineCode = (String)System.Web.HttpContext.Current.Session["TodayList_Condition"];
            }
            viewModel.LineCode = lineCode;

            //本日の変化点の情報を取得
            ChangePointRepository repository = new ChangePointRepository();
            IList<TodayListViewModel> TodayListList = repository.FindTodayList(StaticUtil.ConvDatetime(DateTime.Now.ToString("yyyy-MM-dd")), lineCode);
            foreach (var item in TodayListList)
            {
                //4M区分の表示ラベルを取得する
                item.ChangeType = StaticUtil.GetChangeTypeLable(item.ChangeType);
                item.Comment = item.Comment;
            }
            ViewData["list"] = TodayListList;

            //最終閲覧日時を取得
            UserRepository userRepository = new UserRepository();
            m_user userList = userRepository.FindUser(StaticUtil.getSessionUser().UserId) ;
            viewModel.LastViewDt = userList.last_view_dt;
            IList<string> lineLists = lineList.Select(item => item.Value).ToList();
            viewModel.FinalTimeCount = repository.FindFinalTimeList(viewModel.LastViewDt, lineLists).Count;

            return View(viewModel);

        }

        /// <summary>
        /// ステータス毎の画面遷移
        /// </summary>
        /// <param name="lineCode">ラインコード</param>
        /// <returns></returns>
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult NextAction(TodayListViewModel viewModel)
        {
            // 変化点情報ID
            String changePointId = viewModel.ChangePointId;
            // 変化点情報状態登録
            ChangePointRepository repository = new ChangePointRepository();
            t_change_point changePoint = repository.FindChangePointById(Convert.ToInt32(changePointId));

            // ステータス毎の画面遷移
            String nextAction = "TodayList";
            if (changePoint != null)
            {
                switch (changePoint.deal_status)
                {
                    case "00": //未登録
                        nextAction = "ChangeDetail";
                        break;
                    case "10": // 登録済
                        if (Const.REGISTE_STATUS_REGISTE.Equals(changePoint.training_status))
                        {
                            // 教育結果確認
                            nextAction = "TrainingResult";
                        }
                        else
                        {
                            // 変化点教育登録
                            nextAction = "Training";
                        }
                        break;
                    case "20": // 教育確認NG
                        nextAction = "TrainingResult";
                        break;
                    case "29": // 教育確認済
                        if (String.IsNullOrEmpty(changePoint.line_leader_code))
                        {
                            nextAction = "LeaderConfirm"; // ライン長登録へ
                        }
                        else
                        {
                            nextAction = "CheckResult"; // 検査結果確認 へ
                        }
                        break;
                    case "30": // L長確認待ち
                        nextAction = "LeaderConfirm"; // ライン長登録へ
                        break;
                    case "40": // 検査確認NG
                        nextAction = "CheckResult";
                        break;
                    case "49": // 検査確認済
                        nextAction = "FinalConfirm";
                        break;
                    case "50": // 最終確認NG
                        nextAction = "FinalConfirm";
                        break;
                    case "59": // 最終確認済
                        if (String.IsNullOrEmpty(changePoint.approver_code))
                        {
                            nextAction = "ManagerConfirm"; // 管理者認証へ
                        }
                        else
                        {
                            nextAction = "ManagerApproval"; // 管理者承認 へ
                        }
                        break;
                }
            }
            return RedirectToAction("Index", nextAction, new
            {
                id = viewModel.ChangePointId,
                path = "TodayList"
            });
        }


        #region 工程リストの取得
        /// <summary>
        /// ラインコードに基づく工程一覧を取得する
        /// </summary>
        /// <param name="lineCode">ラインコード</param>
        /// <returns></returns>
        [HttpGet]
        public ActionResult GetTodayListByLine(string lineCode)
        {

            //本日の変化点の情報を取得
            ChangePointRepository repository = new ChangePointRepository();
            IList<TodayListViewModel> TodayListListByLine = repository.FindTodayList(StaticUtil.ConvDatetime(DateTime.Now.ToString("yyyy-MM-dd")), lineCode);
            foreach (var item in TodayListListByLine)
            {
                //4M区分の表示ラベルを取得する
                item.ChangeType = StaticUtil.GetChangeTypeLable(item.ChangeType);
                if (item.ItemContent == null)
                {
                    item.ItemContent = " ";
                }
                if (item.Comment == null)
                {
                    item.Comment = " ";
                }
                if (item.EfuNo == null)
                {
                    item.EfuNo = " ";
                }
            }

            // ライン情報保持
            System.Web.HttpContext.Current.Session["TodayList_Condition"] = lineCode;

            object obj = new { status = "OK", data = TodayListListByLine };

            return Json(obj, JsonRequestBehavior.AllowGet);
        }
        #endregion
    }
}