﻿using ChangePointSystem.Models;
using ChangePointSystem.Repository;
using ChangePointSystem.Util;
using System;
using System.Collections.Generic;
using System.Web.Helpers;
using System.Web.Mvc;

namespace ChangePointSystem.Controllers

{
    [RoutePrefix("Training")]
    public class TrainingController : BaseController
    {
        [HttpGet]
        public ActionResult Index(string id, String path)
        {
            // 変化点情報取得
            ChangePointRepository changePointRepository = new ChangePointRepository();
            ChangePointViewModel changePoint = changePointRepository.FindChangePointViewById(Convert.ToInt32(id));

            // 画面表示View
            TrainingViewModel viewModel = new TrainingViewModel();
            viewModel.ChangePointId = changePoint.ChangePointId;
            viewModel.LineLabel = changePoint.LineLabel;
            viewModel.ProcessLabel = changePoint.ProcessLabel;
            viewModel.ChangeTypeLabel = changePoint.ChangeTypeLabel;

            // 登録済の変化点教育を取得
            TrainingRepository trainingRepository = new TrainingRepository();
            List<t_training> trainingList = trainingRepository.FindTraining(Convert.ToInt32(id));
            List<string> itemCodes = new List<string>();
            foreach (t_training training in trainingList)
            {
                itemCodes.Add(training.item_code);
            }
            viewModel.ItemCodes = itemCodes;
            viewModel.SelectedItem = String.Join(",", itemCodes);
            // 備考設定
            if (trainingList.Count > 0)
            {
                // 教育者
                viewModel.EducatorId = trainingList[0].educator_id;
                viewModel.EducatorName = trainingList[0].educator_name;
                // 受講者
                viewModel.AttendeeId = trainingList[0].attendee_id;
                viewModel.AttendeeName = trainingList[0].attendee_name;
                // 備考
                viewModel.Comment = trainingList[0].comment;
            }

            // 変化点教育リスト
            TrainingItemRepository itemRepository = new TrainingItemRepository();
            ViewBag.ItemList = itemRepository.FindTrainingItemList();

            //担当者WebGridの取得
            ViewBag.gridD = StaticUtil.GetUserGrid(null);
            ViewBag.gridColumnD = StaticUtil.GetUserGridColumn(ViewBag.gridD);

            // 遷移元パス設定
            if (!String.IsNullOrEmpty(path))
            {
                viewModel.SourcePath = path;

            }

            return View(viewModel);

        }

        #region 登録・更新
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Save(TrainingViewModel viewModel)
        {
            int changePointId = Convert.ToInt32(viewModel.ChangePointId);
            // 変化点教育登録
            String[] itemCodes = viewModel.SelectedItem.Split(Const.SPLIT_COMMA_CHAR);
            TrainingRepository trainingRepository = new TrainingRepository();
            t_training training = null;
            if (itemCodes != null && itemCodes.Length > 0)
            {
                // 既存の変化点教育を削除
                List<t_training> traininglList = trainingRepository.FindTraining(changePointId);
                trainingRepository.Delete(traininglList);

                // 選択の変化点教育を登録
                for (int index = 0; index < itemCodes.Length; index++)
                {
                    training = new t_training();
                    training.change_point_id = changePointId;
                    training.item_code = itemCodes[index];
                    training.priority = index + 1;
                    // 教育者、受講者、備考を最初行に登録
                    if (index == 0)
                    {
                        training.educator_id = viewModel.EducatorId;
                        training.educator_name = viewModel.EducatorName;
                        training.attendee_id = viewModel.AttendeeId;
                        training.attendee_name = viewModel.AttendeeName;
                        training.comment = viewModel.Comment;
                    }
                    trainingRepository.Add(training);
                }
            }

            // 変化点情報状態登録
            ChangePointRepository repository = new ChangePointRepository();
            t_change_point changePoint = repository.FindChangePointById(changePointId);
            Boolean isUpdate = false;
            // 対応状況
            /*
            if (Const.DEAL_STATUS_CODE_NOT_REGISTE.Equals(changePoint.deal_status))
            {
                // 「10:登録済」に更新
                changePoint.deal_status = Const.DEAL_STATUS_CODE_REGISTE;
                isUpdate = true;
            }
            */
            // 変化点教育登録状態
            if (Const.REGISTE_STATUS_NOT_REGISTE.Equals(changePoint.training_status))
            {
                // 「1:登録済」に更新
                changePoint.training_status = Const.REGISTE_STATUS_REGISTE;
                isUpdate = true;
            }
            if (isUpdate)
            {
                repository.Update(changePoint);
            }

            // 遷移元を取得
            PathBean pathData = StaticUtil.GetReturnPath(viewModel.SourcePath, "ChangePoint");

            // 変化点情報ID
            return RedirectToAction("Index", pathData.ReturnPath, new
            {
                id = viewModel.ChangePointId,
                path = pathData.SourcePath
            });
        }
        #endregion

        #region 戻る
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Return(TrainingViewModel viewModel)
        {
            // 遷移元を取得
            PathBean pathData = StaticUtil.GetReturnPath(viewModel.SourcePath, "ChangePoint");

            return RedirectToAction("Index", pathData.ReturnPath, new
            {
                id = viewModel.ChangePointId,
                path = pathData.SourcePath
            });
        }
        #endregion
    }
}
