﻿using ChangePointSystem.Models;
using ChangePointSystem.Repository;
using ChangePointSystem.Util;
using NPOI.POIFS.FileSystem;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Web.WebPages;

namespace ChangePointSystem.Controllers

{
    [RoutePrefix("TrainingResult")]
    public class TrainingResultController : BaseController
    {
        [HttpGet]
        public ActionResult Index(string id, String path)

        {
            int changePointIds = Convert.ToInt32(id);

            // 変化点情報取得
            ChangePointRepository changePointRepository = new ChangePointRepository();
            ChangePointViewModel changePoint = changePointRepository.FindChangePointViewById(Convert.ToInt32(id));

            // 画面表示View
            TrainingResultViewModel viewModel = new TrainingResultViewModel();
            TrainingCheckResultRepository TrainingResultRepository = new TrainingCheckResultRepository();
            viewModel = TrainingResultRepository.FindTrainingResultById(changePointIds);

            // 確認結果リスト
            ViewBag.TrainingResultList = SelectListUtil.GetConfirmResultSelectList();

            // 遷移元パス設定
            if (!String.IsNullOrEmpty(path))
            {
                viewModel.SourcePath = path;

            }

            viewModel.ChangePointId = Convert.ToInt32(changePoint.ChangePointId);
            viewModel.LineCode = changePoint.LineCode;
            viewModel.LineLabel = changePoint.LineLabel;
            viewModel.ProcessCode = changePoint.ProcessCode;
            viewModel.ProcessLabel = changePoint.ProcessLabel;
            viewModel.ChangeType = changePoint.ChangeType;
            viewModel.ChangeTypeLabel = changePoint.ChangeTypeLabel;

            if (viewModel != null)
            {
                string[] ReconfirmResult_status1 = new string[6] { viewModel.ReconfirmResult1_0, viewModel.ReconfirmResult1_1, viewModel.ReconfirmResult1_2, viewModel.ReconfirmResult1_3, viewModel.ReconfirmResult1_4, viewModel.ReconfirmResult1_5 };
                string[] ReconfirmResult_status2 = new string[6] { viewModel.ReconfirmResult2_0, viewModel.ReconfirmResult2_1, viewModel.ReconfirmResult2_2, viewModel.ReconfirmResult2_3, viewModel.ReconfirmResult2_4, viewModel.ReconfirmResult2_5 };
                string[] ReconfirmResult_status3 = new string[6] { viewModel.ReconfirmResult3_0, viewModel.ReconfirmResult3_1, viewModel.ReconfirmResult3_2, viewModel.ReconfirmResult3_3, viewModel.ReconfirmResult3_4, viewModel.ReconfirmResult3_5 };

                viewModel.row1 = StaticUtil.CheckRow(ReconfirmResult_status1);
                viewModel.row2 = StaticUtil.CheckRow(ReconfirmResult_status2);
                viewModel.row3 = StaticUtil.CheckRow(ReconfirmResult_status3);
            }

            return View(viewModel);

        }

        #region 登録・更新
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Save(TrainingResultViewModel viewModel)
        {
            int changePointId = Convert.ToInt32(viewModel.ChangePointId);

            // 確認結果リスト
            ViewBag.TrainingResultList = SelectListUtil.GetConfirmResultSelectList();
            //教育確認内容登録
            TrainingCheckResultRepository TrainingResultRepository = new TrainingCheckResultRepository();

            // エフNo利用中確認
            string iputEfuNo = "";
            if(!String.IsNullOrEmpty(viewModel.EfuNo1))
            {
                iputEfuNo = viewModel.EfuNo1.Substring(0, 1);
            } else if (!String.IsNullOrEmpty(viewModel.EfuNo2))
            {
                iputEfuNo = viewModel.EfuNo2.Substring(0, 1);
            } else if (!String.IsNullOrEmpty(viewModel.EfuNo3))
            {
                iputEfuNo = viewModel.EfuNo3.Substring(0, 1);
            }
            List<String> usingEfuList = TrainingResultRepository.FindUsingEfuNoList(changePointId,
                viewModel.LineCode, iputEfuNo);
            if(usingEfuList != null && usingEfuList.Count > 0)
            {
                // 利用中のエフNoある場合、エラーとする
                ViewBag.ErrorMessage = String.Format(Resources.message.E526, 
                    String.Join(Const.SPLIT_COMMA_STRING, iputEfuNo));
                return View("Index", viewModel);
            }

            // 既存の教育確認情報取得
            t_training_check_result TrainingResult = TrainingResultRepository.FindTrainingResult(changePointId);

            if (TrainingResult == null)
            {
                // 教育確認情報新規登録
                TrainingResult = new t_training_check_result()
                {
                    change_point_id = changePointId,
                    training_check_type = Const.TRAINING_TYPE,
                    efu_no1 = viewModel.EfuNo1,
                    product_number1 = viewModel.ProductNumber1,
                    cereal_lot_no1 = viewModel.CerealLotNo1,
                    result1 = viewModel.ReconfirmResult1_0,
                    reconfirm_result1_1 = viewModel.ReconfirmResult1_1,
                    reconfirm_result1_2 = viewModel.ReconfirmResult1_2,
                    reconfirm_result1_3 = viewModel.ReconfirmResult1_3,
                    reconfirm_result1_4 = viewModel.ReconfirmResult1_4,
                    reconfirm_result1_5 = viewModel.ReconfirmResult1_5,
                    efu_no2 = viewModel.EfuNo2,
                    product_number2 = viewModel.ProductNumber2,
                    cereal_lot_no2 = viewModel.CerealLotNo2,
                    result2 = viewModel.ReconfirmResult2_0,
                    reconfirm_result2_1 = viewModel.ReconfirmResult2_1,
                    reconfirm_result2_2 = viewModel.ReconfirmResult2_2,
                    reconfirm_result2_3 = viewModel.ReconfirmResult2_3,
                    reconfirm_result2_4 = viewModel.ReconfirmResult2_4,
                    reconfirm_result2_5 = viewModel.ReconfirmResult2_5,
                    efu_no3 = viewModel.EfuNo3,
                    product_number3 = viewModel.ProductNumber3,
                    cereal_lot_no3 = viewModel.CerealLotNo3,
                    result3 = viewModel.ReconfirmResult3_0,
                    reconfirm_result3_1 = viewModel.ReconfirmResult3_1,
                    reconfirm_result3_2 = viewModel.ReconfirmResult3_2,
                    reconfirm_result3_3 = viewModel.ReconfirmResult3_3,
                    reconfirm_result3_4 = viewModel.ReconfirmResult3_4,
                    reconfirm_result3_5 = viewModel.ReconfirmResult3_5,
                    ng_detail = viewModel.NgDetail,
                    deal_detail = viewModel.DealDetail
                };
                TrainingResultRepository.Add(TrainingResult);
            }
            else
            {
                // 教育確認情報更新
                TrainingResult.change_point_id = changePointId;
                TrainingResult.training_check_type = Const.TRAINING_TYPE;
                TrainingResult.efu_no1 = viewModel.EfuNo1;
                TrainingResult.product_number1 = viewModel.ProductNumber1;
                TrainingResult.result1 = viewModel.ReconfirmResult1_0;
                TrainingResult.reconfirm_result1_1 = viewModel.ReconfirmResult1_1;
                TrainingResult.reconfirm_result1_2 = viewModel.ReconfirmResult1_2;
                TrainingResult.reconfirm_result1_3 = viewModel.ReconfirmResult1_3;
                TrainingResult.reconfirm_result1_4 = viewModel.ReconfirmResult1_4;
                TrainingResult.reconfirm_result1_5 = viewModel.ReconfirmResult1_5;
                TrainingResult.efu_no2 = viewModel.EfuNo2;
                TrainingResult.product_number2 = viewModel.ProductNumber2;
                TrainingResult.result2 = viewModel.ReconfirmResult2_0;
                TrainingResult.reconfirm_result2_1 = viewModel.ReconfirmResult2_1;
                TrainingResult.reconfirm_result2_2 = viewModel.ReconfirmResult2_2;
                TrainingResult.reconfirm_result2_3 = viewModel.ReconfirmResult2_3;
                TrainingResult.reconfirm_result2_4 = viewModel.ReconfirmResult2_4;
                TrainingResult.reconfirm_result2_5 = viewModel.ReconfirmResult2_5;
                TrainingResult.efu_no3 = viewModel.EfuNo3;
                TrainingResult.product_number3 = viewModel.ProductNumber3;
                TrainingResult.result3 = viewModel.ReconfirmResult3_0;
                TrainingResult.reconfirm_result3_1 = viewModel.ReconfirmResult3_1;
                TrainingResult.reconfirm_result3_2 = viewModel.ReconfirmResult3_2;
                TrainingResult.reconfirm_result3_3 = viewModel.ReconfirmResult3_3;
                TrainingResult.reconfirm_result3_4 = viewModel.ReconfirmResult3_4;
                TrainingResult.reconfirm_result3_5 = viewModel.ReconfirmResult3_5;
                TrainingResult.ng_detail = viewModel.NgDetail;
                TrainingResult.deal_detail = viewModel.DealDetail;

                TrainingResultRepository.Update(TrainingResult);
            }

            // 既存の教育確認情報取得
            t_training_check_result CheckResult = TrainingResultRepository.FindCheckResult(changePointId);

            if (CheckResult != null) {
                CheckResult.efu_no1 = viewModel.EfuNo1;
                CheckResult.product_number1 = viewModel.ProductNumber1;
                CheckResult.efu_no2 = viewModel.EfuNo2;
                CheckResult.product_number2 = viewModel.ProductNumber2;
                CheckResult.efu_no3 = viewModel.EfuNo3;
                CheckResult.product_number3 = viewModel.ProductNumber3;
                TrainingResultRepository.Update(CheckResult);
            }


            // 変化点情報状態登録
            ChangePointRepository repository = new ChangePointRepository();
            t_change_point changePoint = repository.FindChangePointById(changePointId);
            Boolean isUpdate = false;

            // 対応状況
            string[][] ReconfirmResult = new string[3][];
            string[] ReconfirmResult_status = new string[3];
            ReconfirmResult[0] = new string[7] { viewModel.EfuNo1, viewModel.ReconfirmResult1_0, viewModel.ReconfirmResult1_1, viewModel.ReconfirmResult1_2, viewModel.ReconfirmResult1_3, viewModel.ReconfirmResult1_4, viewModel.ReconfirmResult1_5 };
            ReconfirmResult[1] = new string[7] { viewModel.EfuNo2, viewModel.ReconfirmResult2_0, viewModel.ReconfirmResult2_1, viewModel.ReconfirmResult2_2, viewModel.ReconfirmResult2_3, viewModel.ReconfirmResult2_4, viewModel.ReconfirmResult2_5 };
            ReconfirmResult[2] = new string[7] { viewModel.EfuNo3, viewModel.ReconfirmResult3_0, viewModel.ReconfirmResult3_1, viewModel.ReconfirmResult3_2, viewModel.ReconfirmResult3_3, viewModel.ReconfirmResult3_4, viewModel.ReconfirmResult3_5 };

            ReconfirmResult_status = StaticUtil.getReconfirmResult(ReconfirmResult);

            if (ReconfirmResult_status.Contains("NO"))
            {

            }
            else if (ReconfirmResult_status.Contains("NG"))
            {
                // 「29:教育確認NG」に更新
                if ((changePoint.deal_status == Const.DEAL_STATUS_CODE_REGISTE ||
                    changePoint.deal_status == Const.DEAL_STATUS_CODE_TRAINING))
                {
                    changePoint.deal_status = Const.DEAL_STATUS_CODE_TRAINING_NG;
                    isUpdate = true;
                }
            }
            else if (ReconfirmResult_status.Contains("OK"))
            {
                if (((changePoint.deal_status == Const.DEAL_STATUS_CODE_REGISTE ||
                    changePoint.deal_status == Const.DEAL_STATUS_CODE_TRAINING_NG)))
                    {

                        // 「教育結果登録状態:登録済」に更新
                        changePoint.training_result_status = Const.REGISTE_STATUS_REGISTE;
                    // 「20:教育確認済」に更新
                    changePoint.deal_status = Const.DEAL_STATUS_CODE_TRAINING;
                    isUpdate = true;
                }
            }

            // 変化内容登録状態
            if (Const.REGISTE_STATUS_NOT_REGISTE.Equals(changePoint.training_result_status))
            {
                // 「1:登録済」に更新
                changePoint.training_result_status = Const.REGISTE_STATUS_REGISTE;
                isUpdate = true;
            }
            // エフNo.登録
            String efuNo = changePoint.efu_no;
            if (!String.IsNullOrEmpty(viewModel.EfuNo1))
            {
                efuNo = viewModel.EfuNo1.Substring(0, 1);
            }
            if (efuNo != changePoint.efu_no)
            {
                changePoint.efu_no = efuNo;
                isUpdate = true;
            }
            if (isUpdate)
            {
                repository.Update(changePoint);
            }

            // 遷移元を取得
            PathBean pathData = StaticUtil.GetReturnPath(viewModel.SourcePath, "ChangePoint");

            // 変化点情報ID
            return RedirectToAction("Index", pathData.ReturnPath, new
            {
                id = viewModel.ChangePointId,
                path = pathData.SourcePath
            });

        }
        #endregion

        #region 戻る
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Return(TrainingResultViewModel viewModel)
        {
            // 遷移元を取得
            PathBean pathData = StaticUtil.GetReturnPath(viewModel.SourcePath, "ChangePoint");

            // 変化点情報ID
            return RedirectToAction("Index", pathData.ReturnPath, new
            {
                id = viewModel.ChangePointId,
                path = pathData.SourcePath
            });
        }
        #endregion
    }
}
