﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Web;
using System.Web.Mvc;

namespace ChangePointSystem.Helpers
{
    /// <summary>
    /// HTMLCheckBox用フィルター
    /// </summary>
    public static class CheckBoxHelpers
    {

        #region CheckBoxListFor
        /// <summary>
        /// SelectListItemリストをチェックボックス
        /// <param name="expression"> SelectListItem型のList </param>
        /// <param name="separator"> 区切り文字 </param>
        /// </summary>
        public static MvcHtmlString CheckBoxListFor<TModel, TProperty, VProperty>(
            this HtmlHelper<TModel> htmlHelper
            , Expression<Func<TModel, TProperty>> expression
            , Expression<Func<TModel, VProperty>> valueModel
            , String separator
            )
        {
            var name = ExpressionHelper.GetExpressionText(expression);

            var fullName = htmlHelper.ViewContext.ViewData.TemplateInfo.GetFullHtmlFieldName(name);

            var metadata = ModelMetadata.FromLambdaExpression<TModel, VProperty>(valueModel, htmlHelper.ViewData);
            
            var values = new List<String>();

            foreach (String obj in (IEnumerable)metadata.Model ?? new object[0])
            {
                values.AddRange(obj.Split(',').ToArray());
                //values.Add(obj);

            }

            var result = new StringBuilder();

            var selectList = (IEnumerable<SelectListItem>)htmlHelper.ViewData.Eval(name);
            
            foreach (var selectListItem in selectList)
            {
                var value = selectListItem.Value ?? selectListItem.Text;
                
                var tId = fullName + "_" + value + "_";

                //input
                var tagBuilder = new TagBuilder("input");

                tagBuilder.MergeAttribute("id", tId);
                tagBuilder.MergeAttribute("type", "checkbox");
                tagBuilder.MergeAttribute("name", ExpressionHelper.GetExpressionText(valueModel), true);
                tagBuilder.MergeAttribute("value", value);

                if (values.Contains(value))
                {
                    tagBuilder.MergeAttribute("checked", "checked");
                }

                result.Append(tagBuilder.ToString(TagRenderMode.SelfClosing));

                //label
                var labelBuilder = new TagBuilder("label");

                labelBuilder.MergeAttribute("for", tId);
                labelBuilder.InnerHtml = selectListItem.Text;

                result.Append(labelBuilder.ToString(TagRenderMode.Normal));
                
                result.Append(separator);
            }

            return MvcHtmlString.Create(result.ToString());
        }
        #endregion

    }
}