﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;

namespace ChangePointSystem.Helpers
{
    /// <summary>
    /// 戻るボタン
    /// </summary>
    public static class ReferrerLinkExtensions
    {
        public static IHtmlString ReferrerLink(
          this HtmlHelper htmlHelper,
          string linkText,
          IDictionary<string, object> htmlAttributes)
        {
            //BaseController > OnActionExecuting で取得している値を取得する
            Uri prev = (Uri)System.Web.HttpContext.Current.Session["CurUrl"];
            if(prev == null) prev = (Uri) htmlHelper.ViewContext.HttpContext.Request.UrlReferrer;
            
            string referrerString = prev.AbsoluteUri;

            //相対パスにする
            int pathIndex = prev.AbsoluteUri.IndexOf(prev.AbsolutePath);
            referrerString = prev.AbsoluteUri.Remove(0, pathIndex);

            if (!(bool)System.Web.HttpContext.Current.Session["isRedirect"])
            {
                //ページの取得
                string page = (string)htmlHelper.ViewContext.HttpContext.Request.RequestContext.RouteData.Values["page"];
                int pgIndex = referrerString.IndexOf("&page=");
                if (pgIndex >= 0)
                {
                    referrerString = referrerString.Remove(pgIndex, referrerString.Length - pgIndex);
                }
                if (page != null && page != "0") referrerString += "&page=" + (Convert.ToInt32(page) + 1);
                if (string.IsNullOrEmpty(referrerString)) return MvcHtmlString.Empty;

            }

            if(referrerString.IndexOf('?') > 0)
            {
                referrerString += "&";
            }
            else
            {
                referrerString += "?";
            }

            //タグの生成
            var tagBuilder = new TagBuilder("button");
            tagBuilder.SetInnerText(linkText);
            tagBuilder.MergeAttributes(htmlAttributes);
            tagBuilder.MergeAttribute("value", "returnBtn");
            tagBuilder.MergeAttribute("name", "returnBtn");
            tagBuilder.MergeAttribute("onclick", "location.href='" + referrerString + "return=true" + "'", true);

            string tagString = tagBuilder.ToString();
            //System.Diagnostics.Debug.WriteLine("戻るボタンURL：" + referrerString);

            return MvcHtmlString.Create(tagString);
        }

        public static IHtmlString ReferrerLink(
          this HtmlHelper htmlHelper,
          string linkText,
          object htmlAttributes)
        {
            var dic = HtmlHelper.AnonymousObjectToHtmlAttributes(htmlAttributes);
            return ReferrerLink(htmlHelper, linkText, dic);
        }

        public static IHtmlString ReferrerLink(
          this HtmlHelper htmlHelper,
          string linkText)
        {
            return ReferrerLink(htmlHelper, linkText, (object)null);
        }




    }
}