﻿using ChangePointSystem.Models;
using ChangePointSystem.Util;
using System;
using System.Collections.Generic;
using System.Linq;

namespace ChangePointSystem.Repository
{
    /// <summary>
    /// 変化点項目マスタ
    /// </summary>
    public class ChangeItemRepository : Repository<m_change_item>
    {

        /// <summary>
        /// 変化点項目を取得し、リスト型で返却する
        /// </summary>
        /// <param name="changeType">4M区分</param>
        /// <param name="beforeAfterType">前後区分</param>
        /// <returns></returns>
        public List<m_change_item> FindChangeItemListByType(String changeType,
            String beforeAfterType)
        {
            var m = db.m_change_item
                .Where(r => r.delete_flg == Const.DELETE_FLG_OFF)
                .Where(r => r.change_type == changeType);
            if(!String.IsNullOrEmpty(beforeAfterType) &&
                !Const.BEFORE_AFTER_OTHER_CODE.Equals(beforeAfterType))
            {
                m = m.Where(r => r.before_after_type == beforeAfterType ||
                r.before_after_type == Const.BEFORE_AFTER_CODE);
            }

            m = m.Distinct().OrderBy(r => r.display_order);

            return m.ToList();
        }

        /// <summary>
        /// 変化点項目を取得し、コード⇒名称のマップで返却する
        /// </summary>
        /// <param name="changeType">4M区分</param>
        /// <returns></returns>
        public Dictionary<String, String> FindChangeItemMapByType(String changeType)
        {
            List<m_change_item> changeItemList = FindChangeItemListByType(changeType, "");
            Dictionary<String, String> changeItemMap = new Dictionary<String, String>();
            foreach (m_change_item item in changeItemList)
            {
                changeItemMap.Add(item.item_content, item.item_code);
            }

            return changeItemMap;
        }
    }
}