﻿using ChangePointSystem.Models;
using ChangePointSystem.Util;
using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using static NPOI.HSSF.Util.HSSFColor;
using System.Web.UI.WebControls;
using NPOI.SS.Formula.Functions;
using System.Collections;
using System.Web.Mvc;
using System.Web.Helpers;
using System.Web.UI.WebControls.Expressions;
using System.Data.Entity.Infrastructure;
using Microsoft.Win32;
using static log4net.Appender.RollingFileAppender;
using Antlr.Runtime.Misc;

namespace ChangePointSystem.Repository
{
    /// <summary>
    /// 変化点情報
    /// </summary>
    public class ChangePointRepository : Repository<t_change_point>
    {


        #region 追加
        /// <summary>
        /// 追加
        /// <param name="entity">追加エンティティ</param>
        /// </summary>
        public override bool Add(t_change_point entity)
        {
            try
            {
                entity.regist_user = StaticUtil.getSessionUser().UserId;
                entity.regist_dt = StaticUtil.GetNow();
                entity.update_user = StaticUtil.getSessionUser().UserId;
                entity.update_dt = StaticUtil.GetNow();
                entity.delete_flg = Const.DELETE_FLG_OFF;

                db.Entry(entity).State = EntityState.Added;
                db.SaveChanges();

                ///////4m deal status change record 20241009///////////////////////////
                using (PlusDbContext ctx = new PlusDbContext())
                {
                    // 检查是否存在指定条件的记录
                    var rec = ctx.DealRecords.FirstOrDefault(dr => dr.change_point_id == entity.change_point_id && dr.deal_status == entity.deal_status);

                    if (rec != null)
                    {

                        rec.update_user = StaticUtil.getSessionUser().UserId;
                        rec.update_dt = StaticUtil.GetNow();
                    }
                    else
                    {
                        DealRecord newRecord = new DealRecord
                        {
                            change_point_id = entity.change_point_id,
                            deal_status = entity.deal_status,
                            update_user = StaticUtil.getSessionUser().UserId,
                            update_dt = StaticUtil.GetNow()
                        };
                        ctx.DealRecords.Add(newRecord);
                    }

                    // 保存更改到数据库
                    ctx.SaveChanges();
                }
                ////////////////////////////////

                return true;
            }
            catch (Exception e)
            {
                logger.Error(StaticUtil.getSessionUser().UserId + " : " + e);
                return false;

            }
        }
        #endregion

        #region 更新
        /// <summary>
        /// 更新
        /// <param name="entity">変更エンティティ</param>
        /// <return> </return>
        /// </summary>
        public override bool Update(t_change_point entity)
        {
            try
            {
                ///////4m deal status change record 20241009///////////////////////////
                using (PlusDbContext ctx = new PlusDbContext())
                {
                    // 检查是否存在指定条件的记录
                    var rec = ctx.DealRecords.FirstOrDefault(dr => dr.change_point_id == entity.change_point_id && dr.deal_status == entity.deal_status);

                    if (rec != null)
                    {

                        rec.update_user = StaticUtil.getSessionUser().UserId;
                        rec.update_dt = StaticUtil.GetNow();
                    }
                    else
                    {
                        DealRecord newRecord = new DealRecord
                        {
                            change_point_id = entity.change_point_id,
                            deal_status = entity.deal_status,
                            update_user = StaticUtil.getSessionUser().UserId,
                            update_dt = StaticUtil.GetNow()
                        };
                        ctx.DealRecords.Add(newRecord);
                    }

                    // 保存更改到数据库
                    ctx.SaveChanges();
                }
                ////////////////////////////////

                entity.update_user = StaticUtil.getSessionUser().UserId;
                entity.update_dt = StaticUtil.GetNow();

                db.Entry(entity).State = EntityState.Modified;
                db.SaveChanges();

                return true;
            }
            catch (Exception e)
            {
                logger.Error(StaticUtil.getSessionUser().UserId + " : " + e);
                return false;

            }
        }
        #endregion

        #region 削除
        /// <summary>
        /// 削除
        /// <param name="entity">削除エンティティ</param>
        /// </summary>
        public override bool Delete(t_change_point entity)
        {

            try
            {
                entity.update_user = StaticUtil.getSessionUser().UserId;
                entity.update_dt = StaticUtil.GetNow();
                entity.delete_flg = Const.DELETE_FLG_ON;

                db.Entry(entity).State = EntityState.Modified;
                db.SaveChanges();

                return true;

            }
            catch (Exception e)
            {
                logger.Error(StaticUtil.getSessionUser().UserId + " : " + e);
                return false;

            }
        }
        #endregion

        #region 変化点情報（単一）
        /// <summary>
        /// 変化点情報IDで、変化点情報を取得
        /// </summary>
        /// <param name="changePointId">変化点情報ID</param>
        /// <returns>変化点情報</returns>
        public t_change_point FindChangePointById(int changePointId)
        {
            var m = db.t_change_point
                .Where(r => r.delete_flg == Const.DELETE_FLG_OFF)
                .Where(r => r.change_point_id == changePointId);

            return m.Distinct().FirstOrDefault();
        }
        #endregion

        #region 変化点表示情報（単一）
        /// <summary>
        /// 変化点情報IDで、変化点表示情報を取得
        /// </summary>
        /// <param name="changePointId">変化点情報ID</param>
        /// <returns>変化点表示情報</returns>
        public ChangePointViewModel FindChangePointViewById(int changePointId)
        {
            var query =
            from point in db.t_change_point
            join line in db.m_line
            on point.line_code equals line.line_code
            join process in db.m_process
            on new { point.line_code, point.process_code } equals new { process.line_code, process.process_code }
            join worker in db.m_worker
            on point.user_id equals worker.work_id
            where point.change_point_id == changePointId
            select new
            {
                point.change_point_id,
                point.manage_no,
                point.accrual_date,
                point.line_code,
                line.line_name,
                point.process_code,
                process.process_name,
                point.user_id,
                worker.work_name,
                point.change_type,
                point.efu_no,
                point.before_after_type,
                point.deal_status,
                point.detail_status,
                point.training_status,
                point.training_result_status,
                point.check_result_status,
                point.confirm_status
            };

            var result = query.Distinct().FirstOrDefault();
            ChangePointViewModel changePointView = new ChangePointViewModel()
            {
                ChangePointId = Convert.ToString(result.change_point_id),
                ManageNo = result.manage_no,
                AccrualDate = result.accrual_date,
                LineCode = result.line_code,
                LineLabel = result.line_name,
                ProcessCode = result.process_code,
                ProcessLabel = result.process_name,
                UserId = result.user_id,
                UserName = result.work_name,
                ChangeType = result.change_type,
                ChangeTypeLabel = StaticUtil.GetChangeTypeLable(result.change_type),
                EfuNo = result.efu_no,
                BeforeAfterType = result.before_after_type,
                BeforeAfterTypeLabel = StaticUtil.GetBeforeAfterLable(result.before_after_type),
                DealStatus = result.deal_status,
                DealStatusLabel = StaticUtil.GetDealStatusLable(result.deal_status),
                DetailStatus = result.detail_status,
                DetailStatusLabel = StaticUtil.GetRetisteStatusLable(result.detail_status),
                TrainingStatus = result.training_status,
                TrainingStatusLabel = StaticUtil.GetRetisteStatusLable(result.training_status),
                TrainingResultStatus = result.training_result_status,
                TrainingResultStatusLabel = StaticUtil.GetRetisteStatusLable(result.training_result_status),
                CheckResultStatus = result.check_result_status,
                CheckResultStatusLabel = StaticUtil.GetRetisteStatusLable(result.check_result_status),
                ConfirmStatus = result.confirm_status,
                ConfirmStatusLabel = StaticUtil.GetRetisteStatusLable(result.confirm_status)
            };

            return changePointView;
        }
        #endregion

        #region 変化点情報（日付指定）
        /// <summary>
        /// 発生日を指定して、変化点情報を取得する
        /// </summary>
        /// <param name="accrualDate">発生日</param>
        /// <returns></returns>
        public List<t_change_point> FindChangePointByDate(DateTime accrualDate)
        {
            var m = db.t_change_point
                .Where(r => r.delete_flg == Const.DELETE_FLG_OFF)
                .Where(r => r.accrual_date == accrualDate);
            m = m.Distinct().OrderBy(r => r.change_point_id);

            return m.ToList();
        }
        #endregion

        #region 変化点情報（検索条件指定）
        /// <summary>
        /// 発生日を指定して、変化点情報を取得する
        /// </summary>
        /// <param name="accrualDate">発生日</param>
        /// <returns></returns>
        public List<t_change_point> FindChangePointByCondition(DateTime accrualDateFrom,
            DateTime accrualDateTo, List<String> statusList)
        {
            var m = db.t_change_point
                .Where(r => r.delete_flg == Const.DELETE_FLG_OFF);
            // 発生日From
            if (accrualDateFrom != null)
            {
                m.Where(r => r.accrual_date >= accrualDateFrom);
            }
            // 発生日From
            if (accrualDateTo != null)
            {
                m.Where(r => r.accrual_date <= accrualDateTo);
            }
            // 対応状況
            if (statusList != null && statusList.Count() > 0)
            {
                m.Where(r => statusList.Contains(r.deal_status));
            }
            m = m.Distinct().OrderBy(r => r.change_point_id);

            return m.ToList();
        }
        #endregion

        #region 変化点情報（本日）
        /// <summary>
        /// 本日の変化点情報を取得する
        /// </summary>
        /// <param name="accrualDate">発生日</param>
        /// <returns></returns>
        public List<TodayListViewModel> FindTodayList(DateTime accrualDate, String lineCode)
        {
            DateTime dateTime = new DateTime(0);
            var query =
            from a in db.t_change_point
            join b in db.t_change_detail.Where(x => x.priority == 1)
            on a.change_point_id equals b.change_point_id into temp
            from b in temp.DefaultIfEmpty()
            join worker in db.m_worker
            on a.user_id equals worker.work_id into temp3
            from worker in temp3.DefaultIfEmpty()
            join e in db.t_training_check_result.Where(x => x.training_check_type == Const.TRAINING_TYPE)
            on a.change_point_id equals e.change_point_id into temp4
            from e in temp4.DefaultIfEmpty()
            join c in db.m_process
            on new { a.line_code, a.process_code } equals new { c.line_code, c.process_code }
            join d in db.m_change_item
            on b.item_code equals d.item_code into temp2
            from d in temp2.DefaultIfEmpty()
            where a.delete_flg == Const.DELETE_FLG_OFF && a.accrual_date <= accrualDate &&
                a.line_code == lineCode && a.deal_status != Const.DEAL_STATUS_CODE_FINAL_APPROVED
            orderby a.deal_status ascending,
                    a.accrual_date descending,
                    a.regist_dt descending
            select new
            {
                a.change_point_id,
                c.process_name,
                a.change_type,
                a.deal_status,
                a.efu_no,
                d.item_content,
                b.comment,
                a.manage_no,
                a.accrual_date,
                worker.work_name,
                regist_dt = e == null? dateTime : e.regist_dt,
            };

            var queryList = query.ToList();
            List<TodayListViewModel> List = new List<TodayListViewModel>();
            int i = 0;
            foreach (var item in queryList)
            {
                List.Insert(i, new TodayListViewModel());
                List[i].ChangePointId = Convert.ToString(item.change_point_id);
                List[i].ProcessCode = item.process_name;
                List[i].ChangeType = item.change_type;
                List[i].DealStatus = item.deal_status;
                List[i].EfuNo = item.efu_no;
                List[i].ItemContent = item.item_content;
                List[i].Comment = item.comment;
                List[i].ManageNo = item.manage_no;
                List[i].AccrualDate = item.accrual_date;
                List[i].UserName = item.work_name;
                List[i].Training_dt = item.regist_dt .Equals(dateTime) ? "" : item.regist_dt.ToString("yyyy/MM/dd HH:mm");
              i++;
            }

            return List;
        }
        #endregion

        #region 変化点情報（最終閲覧日時）
        /// <summary>
        /// 本日の変化点情報を取得する
        /// </summary>
        /// <param name="accrualDate">発生日</param>
        /// <returns></returns>
        public List<t_change_point> FindFinalTimeList(Nullable<System.DateTime> FinalTime, IList<string> lineList)
        {
            if (FinalTime == null)
            {
                FinalTime = DateTime.MinValue;
            }
            DateTime tomorrow = StaticUtil.GetTomorrow();

            var query = db.t_change_point
            .Where(r => r.delete_flg == Const.DELETE_FLG_OFF)
            .Where(r => r.accrual_date >= tomorrow)
            .Where(r => r.regist_dt > FinalTime)
            .Where(r => lineList.Contains(r.line_code))
            .Where(r => r.deal_status != Const.DEAL_STATUS_CODE_FINAL_APPROVED)
            .OrderByDescending(r => r.accrual_date)
            .OrderBy(r => r.deal_status)
            .OrderByDescending(r => r.regist_dt);

            var queryList = query.ToList();
            
            return query.ToList();
        }
        #endregion

        #region 変化点履歴
        /// <summary>
        /// 変化点履歴を取得する
        /// </summary>
        /// <param name="accrualDateFrom">発生日From</param>
        /// <param name="accrualDateTo">発生日To</param>
        /// <param name="lineCode">ライン名</param>
        /// <param name="changetype">4M</param>
        /// <param name="workType">前／後工程</param>
        /// <param name="processCode">工程名</param>
        /// <param name="productNo">品番</param>
        /// <param name="cereal">シリアル</param>
        /// <param name="dealStatus">ステータス</param>
        /// <param name="rangeType">範囲</param>
        /// 
        /// <returns></returns>
        public List<ChangeListViewModel> FindChangeList(
            Nullable<DateTime> accrualDateFrom, 
            Nullable<DateTime> accrualDateTo, 
            String lineCode,
            String changetype,
            String workType,
            String processCode,
            String ProductNumber1,
            String ProductNumber2,
            String ProductNumber3,
            String dealStatus, 
            String rangeType,
            int length,
            IList<string> lineList)
        {
            Boolean lineCodeIsEmpty = lineCode == "";
            Boolean changetypeIsEmpty = changetype == "";
            Boolean workTypeIsEmpty = workType == "";
            Boolean processCodeIsEmpty = processCode == "";

            Boolean less = rangeType == "1";
            Boolean greater = rangeType == "2";
            Boolean equal = rangeType == "3";
            DateTime dateTime = new DateTime(0);


            if (ProductNumber1 == null) 
            {
                ProductNumber1 = "";
            }
            if (ProductNumber2 == null)
            {
                ProductNumber2 = "";
            }
            if (ProductNumber3 == null)
            {
                ProductNumber3 = "";
            }

            Boolean AllProductNumberIsEmpty = ProductNumber1 == "" && ProductNumber2 == "" && ProductNumber3 == "";


            var query =
                from a in db.t_change_point
                join b in db.t_change_detail.Where(x => x.priority == 1)
                    on new { a.change_point_id } equals new { b.change_point_id } into temp
                from b in temp.DefaultIfEmpty()
                join worker in db.m_worker
                    on a.user_id equals worker.work_id into temp7
                from worker in temp7.DefaultIfEmpty()
                join c in db.m_process
                    on new { a.line_code, a.process_code } equals new { c.line_code, c.process_code }
                join d in db.m_change_item
                    on b.item_code equals d.item_code into temp2
                join h in db.m_line
                    on a.line_code equals h.line_code
                from e in temp2.DefaultIfEmpty()
                join f in db.t_training_check_result.Where(x => x.training_check_type == Const.TRAINING_TYPE)
                    on a.change_point_id equals f.change_point_id into temp3
                from f in temp3.DefaultIfEmpty()
                join n in db.t_training_check_result.Where(x => x.training_check_type == Const.CHECK_TYPE2)
                   on a.change_point_id equals n.change_point_id into temp6
                from n in temp6.DefaultIfEmpty()
                join g in db.t_training.Where(x => x.priority == 1)
                    on a.change_point_id equals g.change_point_id into temp4
                from g in temp4.DefaultIfEmpty()
                join i in db.t_final_confirm
                    on a.change_point_id equals i.change_point_id into temp5
                from i in temp5.DefaultIfEmpty()
                where ((accrualDateFrom == null) || (a.accrual_date >= accrualDateFrom))
                        && ((accrualDateTo == null) || (a.accrual_date <= accrualDateTo))
                        && (lineCodeIsEmpty ? lineList.Contains(a.line_code) : a.line_code == lineCode)
                        && ((changetypeIsEmpty) || (a.change_type == changetype))
                        && ((workTypeIsEmpty) || (a.before_after_type == workType))
                        && ((processCodeIsEmpty) || (a.process_code == processCode))
                        && ((less && dealStatus.CompareTo(a.deal_status) >= 0)
                            || (greater && string.Compare(dealStatus, a.deal_status) <= 0)
                            || (equal && a.deal_status == dealStatus))
                        && (((AllProductNumberIsEmpty) || ((f.product_number1.Contains(ProductNumber1) && f.product_number1.Contains(ProductNumber2) && f.product_number1.Contains(ProductNumber3))
                        || (f.product_number2.Contains(ProductNumber1) && f.product_number2.Contains(ProductNumber2) && f.product_number2.Contains(ProductNumber3))
                        || (f.product_number3.Contains(ProductNumber1) && f.product_number3.Contains(ProductNumber2) && f.product_number3.Contains(ProductNumber3)))) 
                        || ((AllProductNumberIsEmpty) || ((n.product_number1.Contains(ProductNumber1) && n.product_number1.Contains(ProductNumber2) && n.product_number1.Contains(ProductNumber3))
                        || (n.product_number2.Contains(ProductNumber1) && n.product_number2.Contains(ProductNumber2) && n.product_number2.Contains(ProductNumber3))
                        || (n.product_number3.Contains(ProductNumber1) && n.product_number3.Contains(ProductNumber2) && n.product_number3.Contains(ProductNumber3)))))
                        && a.delete_flg == Const.DELETE_FLG_OFF
                orderby a.accrual_date descending,
                        a.deal_status ascending,
                        a.regist_dt descending
                select new
                {
                    a.change_point_id,
                    a.accrual_date,
                    a.regist_dt,
                    ChangePointUser = a.regist_user,
                    DetailUser = b.regist_user,
                    TrainCheckUser = f.regist_user,
                    TrainingUser = g.regist_user,
                    FinalUser = i.regist_user,
                    c.process_name,
                    a.change_type,
                    a.deal_status,
                    e.item_content,
                    h.line_name,
                    b.comment,
                    a.efu_no,
                    worker.work_name,
                    t_regist_dt = f == null ? dateTime : f.regist_dt,
                };
            if (length > 0)
            {
                query = query.Take(length);
            }
            var queryList = query.ToList();

            List<ChangeListViewModel> List = queryList.Select(item => new ChangeListViewModel()
            {
                ChangePointId = Convert.ToString(item.change_point_id),
                AccrualDate = item.accrual_date,
                ProcessLabel = item.process_name,
                ChangeType = item.change_type,
                DealStatusLabel = item.deal_status,
                ItemContent = item.item_content,
                LineLabel = item.line_name,
                Comment = item.comment,
                ChangePointUser = item.ChangePointUser,
                DetailUser = item.DetailUser,
                TrainCheckUser = item.TrainCheckUser,
                TrainingUser = item.TrainingUser,
                FinalUser = item.FinalUser,
                RegistDt = item.regist_dt,
                EfuNo=string.IsNullOrEmpty(item.efu_no)?"-": item.efu_no,
                Work_Name=item.work_name,
                Training_dt = item.t_regist_dt.Equals(dateTime) ? "" : item.t_regist_dt.ToString("yyyy/MM/dd HH:mm")
            }).ToList();

            return List;
        }
        #endregion

        #region エフ確認一覧(未取外)
        /// <summary>
        /// 本日の変化点情報を取得する
        /// </summary>
        /// <param name="accrualDate">発生日</param>
        /// <returns></returns>
        public List<EfuListViewModel> FindEfuList(String lineCode)
        {
            string dealStatus = Const.DEAL_STATUS_CODE_CHECK;
            bool greater = Convert.ToInt32(dealStatus) >= Convert.ToInt32(Const.DEAL_STATUS_CODE_CHECK);
            var query =
                from a in db.t_change_point
                join b in db.t_change_detail.Where(x => x.priority == 1)
                on a.change_point_id equals b.change_point_id into temp
                from b in temp.DefaultIfEmpty()
                join c in db.m_process
                on new { a.line_code, a.process_code } equals new { c.line_code, c.process_code }
                join d in db.m_line
                on a.line_code equals d.line_code
                join e in db.t_training_check_result
                on a.change_point_id equals e.change_point_id
                where a.delete_flg == Const.DELETE_FLG_OFF &&
                      a.line_code == lineCode &&
                      e.training_check_type == Const.CHECK_TYPE2 &&
                     (a.deal_status != Const.DEAL_STATUS_CODE_CHECK_NG && (greater && string.Compare(dealStatus, a.deal_status) <= 0)) &&
                     ((e.efu_no1 != null && e.efu_remove_confirm1 == Const.EFU_REMOVE_NOT) || (e.efu_no2 != null && e.efu_remove_confirm2 == Const.EFU_REMOVE_NOT) || (e.efu_no3 != null && e.efu_remove_confirm3 == Const.EFU_REMOVE_NOT)) 
                orderby a.accrual_date ascending,
                        a.deal_status ascending,
                        a.regist_dt ascending
                select new
                {
                    a.change_point_id,
                    a.accrual_date,
                    c.process_name,
                    a.change_type,
                    e.efu_no1,
                    e.product_number1,
                    e.cereal_lot_no1,
                    e.efu_remove_confirm1,
                    e.efu_no2,
                    e.product_number2,
                    e.cereal_lot_no2,
                    e.efu_remove_confirm2,
                    e.efu_no3,
                    e.product_number3,
                    e.cereal_lot_no3,
                    e.efu_remove_confirm3,
                    d.line_name
                };

            var queryList = query.ToList();
            List<EfuListViewModel> List = queryList.Select(item => new EfuListViewModel()
            {
                ChangePointId = Convert.ToString(item.change_point_id),
                AccrualDate = item.accrual_date,
                LineName = item.line_name,
                ProcessCode = item.process_name,
                ChangeType = item.change_type,
                EfuNo1 = item.efu_no1, 
                ProductNumber1 = item.product_number1,
                CerealLotNo1 = item.cereal_lot_no1,
                EfuRemoveConfirm1 = item.efu_remove_confirm1,
                EfuNo2 = item.efu_no2,
                ProductNumber2 = item.product_number2,
                CerealLotNo2 = item.cereal_lot_no2,
                EfuRemoveConfirm2 = item.efu_remove_confirm2,
                EfuNo3 = item.efu_no3,
                ProductNumber3 = item.product_number3,
                CerealLotNo3 = item.cereal_lot_no3,
                EfuRemoveConfirm3 = item.efu_remove_confirm3,
            }).ToList();
            

            return List;
        }
        #endregion

        #region エフ確認一覧(全部)
        /// <summary>
        /// 本日の変化点情報を取得する
        /// </summary>
        /// <param name="accrualDate">発生日</param>
        /// <returns></returns>
        public List<EfuListViewModel> FindAllEfuList(String lineCode)
        {
            string dealStatus = Const.DEAL_STATUS_CODE_CHECK;
            bool greater = Convert.ToInt32(dealStatus) >= Convert.ToInt32(Const.DEAL_STATUS_CODE_CHECK);
            var query =
                from a in db.t_change_point
                join b in db.t_change_detail.Where(x => x.priority == 1)
                on a.change_point_id equals b.change_point_id into temp
                from b in temp.DefaultIfEmpty()
                join c in db.m_process
                on new { a.line_code, a.process_code } equals new { c.line_code, c.process_code }
                join d in db.m_line
                on a.line_code equals d.line_code
                join e in db.t_training_check_result
                on a.change_point_id equals e.change_point_id
                where a.delete_flg == Const.DELETE_FLG_OFF &&
                      a.line_code == lineCode &&
                      e.training_check_type == Const.CHECK_TYPE2 &&
                     (a.deal_status != Const.DEAL_STATUS_CODE_CHECK_NG && (greater && string.Compare(dealStatus, a.deal_status) <= 0))
                orderby a.accrual_date ascending,
                        a.deal_status ascending,
                        a.regist_dt ascending
                select new
                {
                    a.change_point_id,
                    a.accrual_date,
                    c.process_name,
                    a.change_type,
                    e.efu_no1,
                    e.product_number1,
                    e.cereal_lot_no1,
                    e.efu_remove_confirm1,
                    e.efu_no2,
                    e.product_number2,
                    e.cereal_lot_no2,
                    e.efu_remove_confirm2,
                    e.efu_no3,
                    e.product_number3,
                    e.cereal_lot_no3,
                    e.efu_remove_confirm3,
                    d.line_name
                };

            string sqlText = query.ToString();
            var queryList = query.ToList();
            List<EfuListViewModel> List = new List<EfuListViewModel>();
            int i = 0;
            foreach (var item in queryList)
            {
                List.Insert(i, new EfuListViewModel());
                List[i].ChangePointId = Convert.ToString(item.change_point_id);
                List[i].AccrualDate = item.accrual_date;
                List[i].LineName = item.line_name;
                List[i].ProcessCode = item.process_name;
                List[i].ChangeType = item.change_type;
                List[i].EfuNo1 = item.efu_no1;
                List[i].ProductNumber1 = item.product_number1;
                List[i].CerealLotNo1 = item.cereal_lot_no1;
                List[i].EfuRemoveConfirm1 = item.efu_remove_confirm1;
                List[i].EfuNo2 = item.efu_no2;
                List[i].ProductNumber2 = item.product_number2;
                List[i].CerealLotNo2 = item.cereal_lot_no2;
                List[i].EfuRemoveConfirm2 = item.efu_remove_confirm2;
                List[i].EfuNo3 = item.efu_no3;
                List[i].ProductNumber3 = item.product_number3;
                List[i].CerealLotNo3 = item.cereal_lot_no3;
                List[i].EfuRemoveConfirm3 = item.efu_remove_confirm3;
                i++;
            }

            return List;
        }
        #endregion
    }
}