﻿using ChangePointSystem.Models;
using ChangePointSystem.Util;
using System;
using System.Data.Entity;
using System.Linq;

namespace ChangePointSystem.Repository
{
    /// <summary>
    /// 最終確認
    /// </summary>
    public class FinalConfirmRepository : Repository<t_final_confirm>
    {


        #region 追加
        /// <summary>
        /// 追加
        /// <param name="entity">追加エンティティ</param>
        /// </summary>
        public override bool Add(t_final_confirm entity)
        {
            try
            {

                entity.regist_user = StaticUtil.getSessionUser().UserId;
                entity.regist_dt = StaticUtil.GetNow();
                entity.update_user = StaticUtil.getSessionUser().UserId;
                entity.update_dt = StaticUtil.GetNow();
                entity.delete_flg = Const.DELETE_FLG_OFF;

                db.Entry(entity).State = EntityState.Added;
                db.SaveChanges();

                return true;
            }
            catch (Exception e)
            {
                logger.Error(StaticUtil.getSessionUser().UserId + " : " + e);
                return false;

            }
        }
        #endregion

        #region 更新
        /// <summary>
        /// 更新
        /// <param name="entity">変更エンティティ</param>
        /// <return> </return>
        /// </summary>
        public override bool Update(t_final_confirm entity)
        {
            try
            {

                entity.update_user = StaticUtil.getSessionUser().UserId;
                entity.update_dt = StaticUtil.GetNow();

                db.Entry(entity).State = EntityState.Modified;
                db.SaveChanges();

                return true;

            }
            catch (Exception e)
            {
                logger.Error(StaticUtil.getSessionUser().UserId + " : " + e);
                return false;

            }
        }
        #endregion

        #region 削除
        /// <summary>
        /// 削除
        /// <param name="entity">削除エンティティ</param>
        /// </summary>
        public override bool Delete(t_final_confirm entity)
        {

            try
            {
                entity.update_user = StaticUtil.getSessionUser().UserId;
                entity.update_dt = StaticUtil.GetNow();
                entity.delete_flg = Const.DELETE_FLG_ON;

                db.Entry(entity).State = EntityState.Modified;
                db.SaveChanges();

                return true;

            }
            catch (Exception e)
            {
                logger.Error(StaticUtil.getSessionUser().UserId + " : " + e);
                return false;

            }
        }
        #endregion

        #region 最終確認（単一）
        /// <summary>
        /// 変化点情報IDで、最終確認を取得
        /// </summary>
        /// <param name="changePointId">変化点情報ID</param>
        /// <returns>最終確認</returns>
        public t_final_confirm FindFinalConfirmById(int changePointId)
        {
            var m = db.t_final_confirm
                .Where(r => r.delete_flg == Const.DELETE_FLG_OFF)
                .Where(r => r.change_point_id == changePointId);

            return m.Distinct().FirstOrDefault();
        }
        #endregion
    }
}