﻿using ChangePointSystem.Models;
using ChangePointSystem.Util;
using System;
using System.Collections.Generic;
using System.Linq;

namespace ChangePointSystem.Repository
{
    /// <summary>
    /// ラインマスタ
    /// </summary>
    public class LineRepository : Repository<m_line>
    {
        /// <summary>
        /// ラインを取得し、リスト型で返却する
        /// </summary>
        /// <returns></returns>
        public List<m_line> FindAllLineList()
        {
            var m = db.m_line
                .Where(r => r.delete_flg == Const.DELETE_FLG_OFF);

            m = m.Distinct().OrderBy(r => r.display_order);

            return m.ToList();
        }

        /// <summary>
        /// ラインを取得し、コード⇒名称のマップで返却する
        /// </summary>
        /// <returns></returns>
        public Dictionary<String, String> FindAllLineMap()
        {
            List< m_line> lineList = FindAllLineList();
            Dictionary<String, String> lineMap = new Dictionary<String, String>();
            foreach (m_line item in lineList)
            {
                lineMap.Add(item.line_code, item.line_name);
            }

            return lineMap;
        }

        /// <summary>
        /// ラインコードを指定し、ラインを取得
        /// <param name="lineCode">ラインコード</param>
        /// </summary>
        public m_line FindLineByCode(String lineCode)
        {
            var m = db.m_line.Select(r => r)
                .Where(r => r.line_code == lineCode)
                .Where(r => r.delete_flg == Const.DELETE_FLG_OFF);

            return m.Distinct().FirstOrDefault();
        }

        /// <summary>
        ///組織を指定して、ラインを取得し、リスト型で返却する
        /// </summary>
        /// <returns></returns>
        public List<m_line> FindOrganizationLineMap(List<String> organizationCodeList)
        {
            var m = db.m_line
               .Where(r => r.delete_flg == Const.DELETE_FLG_OFF)
               .Where(r => organizationCodeList.Contains(r.organization_code));

            m = m.Distinct().OrderBy(r => r.display_order);

            return m.ToList();
        }

        /// <summary>
        ///UserNameを指定して、ラインを取得し、リスト型で返却する
        /// </summary>
        /// <returns></returns>
        public List<m_line> FindLineListByUserId(string userId)
        {
            var orgCode = db.m_user
                            .Where(u => u.user_id == userId && u.delete_flg == Const.DELETE_FLG_OFF)
                            .Join(db.m_organization, u => u.organization_code, o => o.organization_code, (u, o) => o.organization_code)
                            .FirstOrDefault();

            var lines = db.m_line
                        .Where(l => l.organization_code == orgCode && l.delete_flg == Const.DELETE_FLG_OFF)
                        .Distinct()
                        .OrderBy(l => l.display_order)
                        .ToList();

            return lines;
        }

    }
}