﻿using ChangePointSystem.Models;
using ChangePointSystem.Util;
using System;
using System.Data.Entity;
using System.Linq;

namespace ChangePointSystem.Repository
{
    /// <summary>
    /// 採番マスタ
    /// </summary>
    public class NumberRepository : Repository<m_number>
    {


        #region 追加
        /// <summary>
        /// 追加
        /// <param name="entity">追加エンティティ</param>
        /// </summary>
        public override bool Add(m_number entity)
        {
            try
            {

                entity.regist_user = StaticUtil.getSessionUser().UserId;
                entity.regist_dt = StaticUtil.GetNow();
                entity.update_user = StaticUtil.getSessionUser().UserId;
                entity.update_dt = StaticUtil.GetNow();
                entity.delete_flg = Const.DELETE_FLG_OFF;

                db.Entry(entity).State = EntityState.Added;
                db.SaveChanges();

                return true;
            }
            catch (Exception e)
            {
                logger.Error(StaticUtil.getSessionUser().UserId + " : " + e);
                return false;

            }
        }
        #endregion

        #region 更新
        /// <summary>
        /// 更新
        /// <param name="entity">変更エンティティ</param>
        /// <return> </return>
        /// </summary>
        public override bool Update(m_number entity)
        {
            try
            {

                entity.update_user = StaticUtil.getSessionUser().UserId;
                entity.update_dt = StaticUtil.GetNow();

                db.Entry(entity).State = EntityState.Modified;
                db.SaveChanges();

                return true;

            }
            catch (Exception e)
            {
                logger.Error(StaticUtil.getSessionUser().UserId + " : " + e);
                return false;
            }
        }
        #endregion

        #region 採番情報取得
        /// <summary>
        /// 採番マスタキー情報に基づく採番データを取得
        /// </summary>
        /// <param name="numberType">採番区分</param>
        /// <param name="numberKey">採番キー</param>
        /// <param name="numberUnit">採番単位</param>
        /// <returns>採番データ</returns>
        public m_number FindNumber(String numberType, String numberKey, String numberUnit)
        {
            var m = db.m_number
                .Where(r => r.delete_flg == Const.DELETE_FLG_OFF)
                .Where(r => r.number_type == numberType)
                .Where(r => r.number_key == numberKey)
                .Where(r => r.number_unit == numberUnit);

            return m.Distinct().FirstOrDefault();
        }
        #endregion
    }
}