﻿using ChangePointSystem.Models;
using ChangePointSystem.Util;
using System;
using System.Collections.Generic;
using System.Linq;

namespace ChangePointSystem.Repository
{
    /// <summary>
    /// ラインマスタ
    /// </summary>
    public class OrganizationRepository : Repository<m_line>
    {
        /// <summary>
        /// ラインを取得し、リスト型で返却する
        /// </summary>
        /// <returns></returns>
        public List<m_organization> FindAllOrganizationList()
        {
            var m = db.m_organization
                .Where(r => r.delete_flg == Const.DELETE_FLG_OFF);

            m = m.Distinct();

            return m.ToList();
        }

        /// <summary>
        /// ラインを取得し、コード⇒名称のマップで返却する
        /// </summary>
        /// <returns></returns>
        public Dictionary<String, String> FindAllOrganizationMap()
        {
            List<m_organization> OrganizationList = FindAllOrganizationList();
            Dictionary<String, String> lineMap = new Dictionary<String, String>();
            foreach (m_organization item in OrganizationList)
            {
                lineMap.Add(item.organization_code, item.organization_name);
            }

            return lineMap;
        }

        /// <summary>
        /// 下位組織を取得し、リスト型で返却する
        /// </summary>
        /// <param name="organizationCodeList">組織コードリスト</param>
        /// <returns></returns>
        public List<m_organization> FindSubOrganizationList(List<String> organizationCodeList)
        {
            var m = db.m_organization
                .Where(r => organizationCodeList.Contains(r.parent_organization_code))
                .Where(r => r.delete_flg == Const.DELETE_FLG_OFF);

            m = m.Distinct();

            return m.ToList();
        }
    }
}