﻿using ChangePointSystem.Models;
using ChangePointSystem.Util;
using System;
using System.Collections.Generic;
using System.Linq;

namespace ChangePointSystem.Repository
{
    /// <summary>
    /// 工程マスタ
    /// </summary>
    public class ProcessRepository : Repository<m_process>
    {
        /// <summary>
        /// 工程を取得し、リスト型で返却する
        /// </summary>
        /// <returns></returns>
        public List<m_process> FindAllProcessList()
        {
            var m = db.m_process
                .Where(r => r.delete_flg == Const.DELETE_FLG_OFF);

            m = m.Distinct().OrderBy(r => r.display_order);

            return m.ToList();
        }

        /// <summary>
        /// ラインの工程を取得し、リスト型で返却する
        /// </summary>
        /// <param name="lineCode">ラインコード</param>
        /// <returns></returns>
        public List<m_process> FindProcessListByLine(String lineCode)
        {
            var m = db.m_process
                .Where(r => r.delete_flg == Const.DELETE_FLG_OFF)
                .Where(r => r.line_code == lineCode);

            m = m.Distinct().OrderBy(r => r.display_order);

            return m.ToList();
        }

        /// <summary>
        /// ラインの工程を取得し、コード⇒名称のマップで返却する
        /// </summary>
        /// <param name="lineCode">ラインコード</param>
        /// <returns></returns>
        public Dictionary<String, String> FindLineMapByLine(String lineCode)
        {
            List<m_process> processList = FindProcessListByLine(lineCode);
            Dictionary<String, String> processMap = new Dictionary<String, String>();
            foreach (m_process item in processList)
            {
                processMap.Add(item.process_code, item.process_name);
            }

            return processMap;
        }
    }
}