﻿using ChangePointSystem.Models;
using ChangePointSystem.Util;
using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using static NPOI.HSSF.Util.HSSFColor;
using System.Web.UI.WebControls;
using NPOI.SS.Formula.Functions;
using System.Collections;
using System.Web.Mvc;
using System.Web.Helpers;
using System.Data.Entity.Infrastructure;
using Microsoft.Ajax.Utilities;

namespace ChangePointSystem.Repository
{
    /// <summary>
    /// 変化点情報
    /// </summary>
    public class TrainingCheckResultRepository : Repository<t_training_check_result>
    {


        #region 追加
        /// <summary>
        /// 追加
        /// <param name="entity">追加エンティティ</param>
        /// </summary>
        public override bool Add(t_training_check_result entity)
        {
            try
            {

                entity.regist_user = StaticUtil.getSessionUser().UserId;
                entity.regist_dt = StaticUtil.GetNow();
                entity.update_user = StaticUtil.getSessionUser().UserId;
                entity.update_dt = StaticUtil.GetNow();
                entity.delete_flg = Const.DELETE_FLG_OFF;


                db.Entry(entity).State = EntityState.Added;
                db.SaveChanges();

                return true;
            }
            catch (Exception e)
            {
                logger.Error(StaticUtil.getSessionUser().UserId + " : " + e);
                return false;

            }
        }
        #endregion

        #region 更新
        /// <summary>
        /// 更新
        /// <param name="entity">変更エンティティ</param>
        /// <return> </return>
        /// </summary>
        public override bool Update(t_training_check_result entity)
        {
            try
            {

                entity.update_user = StaticUtil.getSessionUser().UserId;
                entity.update_dt = StaticUtil.GetNow();
                entity.delete_flg = Const.DELETE_FLG_OFF;

                db.Entry(entity).State = EntityState.Modified;
                db.SaveChanges();

                return true;

            }
            catch (Exception e)
            {
                logger.Error(StaticUtil.getSessionUser().UserId + " : " + e);
                return false;

            }
        }
        #endregion

        #region 削除
        /// <summary>
        /// 削除
        /// <param name="entity">削除エンティティ</param>
        /// </summary>
        public override bool Delete(List<t_training_check_result> entities)
        {

            try
            {
                db.t_training_check_result.RemoveRange(entities);
                db.SaveChanges();

                return true;

            }
            catch (Exception e)
            {
                logger.Error(StaticUtil.getSessionUser().UserId + " : " + e);
                return false;

            }
        }
        #endregion

        #region 教育確認(viewModel)
        /// <summary>
        /// 変化点情報IDで、教育確認情報を取得
        /// </summary>
        /// <param name="changePointId">教育確認情報</param>
        /// <returns>教育確認情報</returns>
        public TrainingResultViewModel FindTrainingResultById(int changePointId)
        {
            var query = db.t_training_check_result
                .Where(r => r.delete_flg == Const.DELETE_FLG_OFF)
                .Where(r => r.change_point_id == changePointId)
                .Where(r => r.training_check_type == Const.TRAINING_TYPE);

            var result = query.Distinct().FirstOrDefault();
            TrainingResultViewModel viewModel = new TrainingResultViewModel();
            if (query.ToList().Count > 0)
            {
                viewModel.ChangePointId = result.change_point_id;
                viewModel.EfuNo1 = result.efu_no1;
                viewModel.ProductNumber1 = result.product_number1;
                viewModel.CerealLotNo1 = result.cereal_lot_no1;
                viewModel.ReconfirmResult1_0 = result.result1;
                viewModel.ConfirmUserId1 = result.confirm_user_id1;
                viewModel.ReconfirmResult1_1 = result.reconfirm_result1_1;
                viewModel.ReconfirmUserId1_1 = result.reconfirm_user_id1_1;
                viewModel.ReconfirmResult1_2 = result.reconfirm_result1_2;
                viewModel.ReconfirmUserId1_2 = result.reconfirm_user_id1_2;
                viewModel.ReconfirmResult1_3 = result.reconfirm_result1_3;
                viewModel.ReconfirmUserId1_3 = result.reconfirm_user_id1_3;
                viewModel.ReconfirmResult1_4 = result.reconfirm_result1_4;
                viewModel.ReconfirmUserId1_4 = result.reconfirm_user_id1_4;
                viewModel.ReconfirmResult1_5 = result.reconfirm_result1_5;
                viewModel.ReconfirmUserId1_5 = result.reconfirm_user_id1_5;
                viewModel.EfuNo2 = result.efu_no2;
                viewModel.ProductNumber2 = result.product_number2;
                viewModel.CerealLotNo2 = result.cereal_lot_no2;
                viewModel.ReconfirmResult2_0 = result.result2;
                viewModel.ConfirmUserId1 = result.confirm_user_id2;
                viewModel.ReconfirmResult2_1 = result.reconfirm_result2_1;
                viewModel.ReconfirmUserId2_1 = result.reconfirm_user_id2_1;
                viewModel.ReconfirmResult2_2 = result.reconfirm_result2_2;
                viewModel.ReconfirmUserId2_2 = result.reconfirm_user_id2_2;
                viewModel.ReconfirmResult2_3 = result.reconfirm_result2_3;
                viewModel.ReconfirmUserId2_3 = result.reconfirm_user_id2_3;
                viewModel.ReconfirmResult2_4 = result.reconfirm_result2_4;
                viewModel.ReconfirmUserId2_4 = result.reconfirm_user_id2_4;
                viewModel.ReconfirmResult2_5 = result.reconfirm_result2_5;
                viewModel.ReconfirmUserId2_5 = result.reconfirm_user_id2_5;
                viewModel.EfuNo3 = result.efu_no3;
                viewModel.ProductNumber3 = result.product_number3;
                viewModel.CerealLotNo3 = result.cereal_lot_no3;
                viewModel.ReconfirmResult3_0 = result.result3;
                viewModel.ConfirmUserId3 = result.confirm_user_id1;
                viewModel.ReconfirmResult3_1 = result.reconfirm_result3_1;
                viewModel.ReconfirmUserId3_1 = result.reconfirm_user_id3_1;
                viewModel.ReconfirmResult3_2 = result.reconfirm_result3_2;
                viewModel.ReconfirmUserId3_2 = result.reconfirm_user_id3_2;
                viewModel.ReconfirmResult3_3 = result.reconfirm_result3_3;
                viewModel.ReconfirmUserId3_3 = result.reconfirm_user_id3_3;
                viewModel.ReconfirmResult3_4 = result.reconfirm_result3_4;
                viewModel.ReconfirmUserId3_4 = result.reconfirm_user_id3_4;
                viewModel.ReconfirmResult3_5 = result.reconfirm_result3_5;
                viewModel.ReconfirmUserId3_5 = result.reconfirm_user_id3_5;
                viewModel.NgDetail = result.ng_detail;
                viewModel.DealDetail = result.deal_detail;
            }
            

            return viewModel;
        }
        #endregion

        #region 教育確認(List<t_training_check_result>)
        /// <summary>
        /// 変化点情報IDで、教育確認情報を取得
        /// </summary>
        /// <param name="changePointId">教育確認情報</param>
        /// <returns>教育確認情報</returns>
        public t_training_check_result FindTrainingResult(int changePointId)
        {
            var query = db.t_training_check_result
                .Where(r => r.delete_flg == Const.DELETE_FLG_OFF)
                .Where(r => r.change_point_id == changePointId)
                .Where(r => r.training_check_type == Const.TRAINING_TYPE);



            return query.Distinct().FirstOrDefault();
        }
        #endregion

        #region 検査確認(viewModel)
        /// <summary>
        /// 変化点情報IDで、教育確認情報を取得
        /// </summary>
        /// <param name="changePointId">教育確認情報</param>
        /// <returns>教育確認情報</returns>
        public CheckResultViewModel FindCheckResultById(int changePointId)
        {
            var query = db.t_training_check_result
                .Where(r => r.delete_flg == Const.DELETE_FLG_OFF)
                .Where(r => r.change_point_id == changePointId)
                .Where(r => r.training_check_type == Const.CHECK_TYPE2);

            var result = query.Distinct().FirstOrDefault();
            CheckResultViewModel viewModel = null;
            if (query.ToList().Count > 0)
            {
                viewModel = new CheckResultViewModel();
                viewModel.ChangePointId = result.change_point_id;
                viewModel.EfuNo1 = result.efu_no1;
                viewModel.ProductNumber1 = result.product_number1;
                viewModel.CerealLotNo1 = result.cereal_lot_no1;
                viewModel.ReconfirmResult1_0 = result.result1;
                viewModel.ConfirmUserName1 = result.confirm_user_name1;
                viewModel.ReconfirmResult1_1 = result.reconfirm_result1_1;
                viewModel.ReconfirmUserName1_1 = result.reconfirm_user_name1_1;
                viewModel.ReconfirmResult1_2 = result.reconfirm_result1_2;
                viewModel.ReconfirmUserName1_2 = result.reconfirm_user_name1_2;
                viewModel.ReconfirmResult1_3 = result.reconfirm_result1_3;
                viewModel.ReconfirmUserName1_3 = result.reconfirm_user_name1_3;
                viewModel.ReconfirmResult1_4 = result.reconfirm_result1_4;
                viewModel.ReconfirmUserName1_4 = result.reconfirm_user_name1_4;
                viewModel.ReconfirmResult1_5 = result.reconfirm_result1_5;
                viewModel.ReconfirmUserName1_5 = result.reconfirm_user_name1_5;
                viewModel.EfuNo2 = result.efu_no2;
                viewModel.ProductNumber2 = result.product_number2;
                viewModel.CerealLotNo2 = result.cereal_lot_no2;
                viewModel.ReconfirmResult2_0 = result.result2;
                viewModel.ConfirmUserName2 = result.confirm_user_name2;
                viewModel.ReconfirmResult2_1 = result.reconfirm_result2_1;
                viewModel.ReconfirmUserName2_1 = result.reconfirm_user_name2_1;
                viewModel.ReconfirmResult2_2 = result.reconfirm_result2_2;
                viewModel.ReconfirmUserName2_2 = result.reconfirm_user_name2_2;
                viewModel.ReconfirmResult2_3 = result.reconfirm_result2_3;
                viewModel.ReconfirmUserName2_3 = result.reconfirm_user_name2_3;
                viewModel.ReconfirmResult2_4 = result.reconfirm_result2_4;
                viewModel.ReconfirmUserName2_4 = result.reconfirm_user_name2_4;
                viewModel.ReconfirmResult2_5 = result.reconfirm_result2_5;
                viewModel.ReconfirmUserName2_5 = result.reconfirm_user_name2_5;
                viewModel.EfuNo3 = result.efu_no3;
                viewModel.ProductNumber3 = result.product_number3;
                viewModel.CerealLotNo3 = result.cereal_lot_no3;
                viewModel.ReconfirmResult3_0 = result.result3;
                viewModel.ConfirmUserName3 = result.confirm_user_name3;
                viewModel.ReconfirmResult3_1 = result.reconfirm_result3_1;
                viewModel.ReconfirmUserName3_1 = result.reconfirm_user_name3_1;
                viewModel.ReconfirmResult3_2 = result.reconfirm_result3_2;
                viewModel.ReconfirmUserName3_2 = result.reconfirm_user_name3_2;
                viewModel.ReconfirmResult3_3 = result.reconfirm_result3_3;
                viewModel.ReconfirmUserName3_3 = result.reconfirm_user_name3_3;
                viewModel.ReconfirmResult3_4 = result.reconfirm_result3_4;
                viewModel.ReconfirmUserName3_4 = result.reconfirm_user_name3_4;
                viewModel.ReconfirmResult3_5 = result.reconfirm_result3_5;
                viewModel.ReconfirmUserName3_5 = result.reconfirm_user_name3_5;
                viewModel.NgDetail = result.ng_detail;
                viewModel.DealDetail = result.deal_detail;
            }


            return viewModel;
        }
        #endregion

        #region 検査確認(List<t_training_check_result>)
        /// <summary>
        /// 変化点情報IDで、教育確認情報を取得
        /// </summary>
        /// <param name="changePointId">教育確認情報</param>
        /// <returns>教育確認情報</returns>
        public t_training_check_result FindCheckResult(int changePointId)
        {
            var query = db.t_training_check_result
                .Where(r => r.delete_flg == Const.DELETE_FLG_OFF)
                .Where(r => r.change_point_id == changePointId)
                .Where(r => r.training_check_type == Const.CHECK_TYPE2);



            return query.Distinct().FirstOrDefault();
        }
        #endregion

        #region EfuNoで、検査確認情報を取得
        /// <summary>
        /// 変化点情報IDで、教育確認情報を取得
        /// </summary>
        /// <param name="changePointId">教育確認情報</param>
        /// <returns>教育確認情報</returns>
        public t_training_check_result FindCheckResultByEfuNo1(string lineCode,string Efulist)
        {
            var query = 
                from a in db.t_training_check_result
                join b in db.t_change_point
                on  a.change_point_id equals b.change_point_id
                where a.delete_flg == Const.DELETE_FLG_OFF &&
                b.delete_flg == Const.DELETE_FLG_OFF &&
                a.training_check_type == Const.CHECK_TYPE2 && b.line_code == lineCode &&
                ((a.efu_remove_confirm1 == Const.EFU_REMOVE_NOT && a.efu_no1 == Efulist) || (a.efu_remove_confirm2 == Const.EFU_REMOVE_NOT && a.efu_no2 == Efulist) || (a.efu_remove_confirm3 == Const.EFU_REMOVE_NOT && a.efu_no3 == Efulist)) 
                select a;
           var query1 = query.OrderByDescending(item => item.update_dt).FirstOrDefault();

            return query1;
        }
        #endregion

        #region 利用中エフNo取得
        /// <summary>
        /// エフNoリストに、利用中のエフNoを取得する
        /// </summary>
        /// <param name="changePointId">変化点情報ID</param>
        /// <param name="lineCode">ラインコード</param>
        /// <param name="efuNo">エフNo</param>
        /// <returns>利用中エフNoリスト</returns>
        public List<String> FindUsingEfuNoList(int changePointId, string lineCode, string efuNo)
        {
            var query =
                from a in db.t_training_check_result
                join b in db.t_change_point
                on a.change_point_id equals b.change_point_id
                where a.delete_flg == Const.DELETE_FLG_OFF &&
                b.delete_flg == Const.DELETE_FLG_OFF &&
                a.training_check_type == Const.TRAINING_TYPE &&
                a.change_point_id != changePointId &&
                ((a.efu_no1 != null && a.efu_no1.StartsWith(efuNo)) ||
                 (a.efu_no2 != null && a.efu_no2.StartsWith(efuNo)) ||
                 (a.efu_no3 != null && a.efu_no3.StartsWith(efuNo))) &&
                b.line_code == lineCode &&
                b.deal_status != Const.DEAL_STATUS_CODE_FINAL_APPROVED
                select a;

            var queryList = query.ToList();
            List<String> usingEfuList = new List<String>();
            foreach (var item in queryList)
            {
                usingEfuList.Add(item.efu_no1);
                usingEfuList.Add(item.efu_no2);
                usingEfuList.Add(item.efu_no3);
            }

            return usingEfuList;
        }
        #endregion

        #region 更新
        /// <summary>
        /// 更新
        /// <param name="entity">変更エンティティ</param>
        /// <return> </return>
        /// </summary>
        public bool efuNo1Reflection(t_training_check_result entity)
        {
            try
            {

                entity.update_user = StaticUtil.getSessionUser().UserId;
                entity.update_dt = StaticUtil.GetNow();
                entity.delete_flg = Const.DELETE_FLG_OFF;

                db.Entry(entity).State = EntityState.Modified;
                db.SaveChanges();

                return true;

            }
            catch (Exception e)
            {
                logger.Error(StaticUtil.getSessionUser().UserId + " : " + e);
                return false;

            }
        }
        #endregion

    }
}