﻿using ChangePointSystem.Models;
using ChangePointSystem.Util;
using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Web.UI.WebControls;

namespace ChangePointSystem.Repository
{
    /// <summary>
    /// 変化点教育情報
    /// </summary>
    public class TrainingRepository : Repository<t_training>
    {


        #region 追加
        /// <summary>
        /// 追加
        /// <param name="entity">追加エンティティ</param>
        /// </summary>
        public override bool Add(t_training entity)
        {
            try
            {

                entity.regist_user = StaticUtil.getSessionUser().UserId;
                entity.regist_dt = StaticUtil.GetNow();
                entity.update_user = StaticUtil.getSessionUser().UserId;
                entity.update_dt = StaticUtil.GetNow();
                entity.delete_flg = Const.DELETE_FLG_OFF;

                db.Entry(entity).State = EntityState.Added;
                db.SaveChanges();

                return true;
            }
            catch (Exception e)
            {
                logger.Error(StaticUtil.getSessionUser().UserId + " : " + e);
                return false;

            }
        }
        #endregion

        #region 削除
        /// <summary>
        /// 削除
        /// <param name="entity">削除エンティティ</param>
        /// </summary>
        public override bool Delete(List<t_training> entities)
        {

            try
            {
                db.t_training.RemoveRange(entities);
                db.SaveChanges();

                return true;

            }
            catch (Exception e)
            {
                logger.Error(StaticUtil.getSessionUser().UserId + " : " + e);
                return false;

            }
        }
        #endregion

        #region 変化点教育（複数）
        /// <summary>
        /// 変化点情報IDで、変化点教育を取得
        /// </summary>
        /// <param name="changePointId">変化点情報ID</param>
        /// <returns>変化点教育</returns>
        public List<t_training> FindTraining(int changePointId)
        {
            var m = db.t_training
                .Where(r => r.delete_flg == Const.DELETE_FLG_OFF)
                .Where(r => r.change_point_id == changePointId);
            m = m.Distinct().OrderBy(r => r.priority);

            return m.ToList();
        }
        #endregion
    }
}