﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Web.Routing;

namespace ChangePointSystem.Util
{
    public static class ControllerHtml
    {
        private class ViewDataContainer : IViewDataContainer
        {
            public ViewDataContainer(ViewDataDictionary viewData)
            {
                ViewData = viewData;
            }

            public ViewDataDictionary ViewData { get; set; }
        }

        private static HtmlHelper htmlHelper;
        private static AjaxHelper ajaxHelper;

        public static HtmlHelper Html(Controller controller)
        {
            var vdd = new ViewDataDictionary();
            var tdd = new TempDataDictionary();
            var controllerContext = controller.ControllerContext;
            var view = new RazorView(controllerContext, "/", "/", false, null);
            htmlHelper = new HtmlHelper(new ViewContext(controllerContext, view, vdd, tdd, new StringWriter()),
                    new ViewDataContainer(vdd), RouteTable.Routes);

            return htmlHelper;
        }

        public static HtmlHelper Html(Controller controller, object model)
        {
            if (htmlHelper == null || htmlHelper.ViewData.Model == null || !htmlHelper.ViewData.Model.Equals(model))
            {
                var vdd = new ViewDataDictionary();
                vdd.Model = model;
                var tdd = new TempDataDictionary();
                var controllerContext = controller.ControllerContext;
                var view = new RazorView(controllerContext, "/", "/", false, null);
                htmlHelper = new HtmlHelper(new ViewContext(controllerContext, view, vdd, tdd, new StringWriter()),
                     new ViewDataContainer(vdd), RouteTable.Routes);
            }
            return htmlHelper;
        }

        #region AjaxHelperの取得
        /// <summary>
        /// AjaxHelperの取得
        /// <param name="controller"> controller </param>
        /// <return> AjaxHelper </return>
        /// </summary>
        public static AjaxHelper Ajax(Controller controller)
        {
            var vdd = new ViewDataDictionary();
            var tdd = new TempDataDictionary();
            var controllerContext = controller.ControllerContext;
            var view = new RazorView(controllerContext, "/", "/", false, null);
            ajaxHelper = new AjaxHelper(new ViewContext(controllerContext, view, vdd, tdd, new StringWriter()),
                    new ViewDataContainer(vdd), RouteTable.Routes);

            return ajaxHelper;
        }
        #endregion

    }
}