﻿using ChangePointSystem.Models;
using ChangePointSystem.Repository;
using System;
using System.Collections.Generic;
using System.IO;
using System.Text.RegularExpressions;
using System.Web.Helpers;
using System.Web.SessionState;
using System.Web.WebPages;

namespace ChangePointSystem.Util
{
    /// <summary>
    /// Utilクラス
    /// </summary>
    public static class StaticUtil
    {

        /// <summary>
        /// 現在日付の取得
        /// </summary>
        public static DateTime GetToday()
        {
            return DateTime.Today;
        }

        /// <summary>
        /// 現在日付の取得（0:00:00）
        /// </summary>
        public static DateTime GetTodayStart()
        {
            return StaticUtil.ConvStartDatetime(StaticUtil.GetTodayString().Substring(0,10));
        }

        /// <summary>
        /// 現在日付の取得（23:59:59）
        /// </summary>
        public static DateTime GetTodayEnd()
        {
            return StaticUtil.ConvEndDatetime(StaticUtil.GetTodayString().Substring(0, 10));
        }

        /// <summary>
        /// 現在日付の取得（String型）
        /// </summary>
        public static String GetTodayString()
        {
            return DateTime.Today.ToString();
        }

        /// <summary>
        /// 現在日時の取得
        /// </summary>
        public static DateTime GetNow()
        {
            return DateTime.Now;
        }

        /// <summary>
        /// 翌日日付の取得
        /// </summary>
        public static DateTime GetTomorrow()
        {
            return DateTime.Today.AddDays(1);
        }

        /// <summary>
        /// 日時が一致しているかどうかを返す
        /// <param name="dt1">比較する日付1</param>
        /// <param name="dt1">比較する日付2</param>
        /// <return>true:一致、false:不一致</return>
        /// </summary>
        public static bool Comparison(DateTime? dt1, DateTime? dt2)
        {
            //nullの場合は初期化する
            DateTime dt1_ = dt1 == null ? default(DateTime) : (DateTime)dt1;
            DateTime dt2_ = dt2 == null ? default(DateTime) : (DateTime)dt2;

            //1秒以下は一致
            return Math.Abs((dt1_ - dt2_).TotalSeconds) < 1;
        }

        /// <summary>
        /// 文字列の末尾から指定した長さの文字列を取得する
        /// </summary>
        /// <param name="str">文字列</param>
        /// <param name="len">長さ</param>
        /// <returns>取得した文字列</returns>
        public static string Right(string str, int len)
        {
            if (len < 0)
            {
                throw new ArgumentException("引数'len'は0以上でなければなりません。");
            }
            if (str == null)
            {
                return "";
            }
            if (str.Length <= len)
            {
                return str;
            }
            return str.Substring(str.Length - len, len);
        }
        /// <summary>
        /// 数値変換
        /// <param name="str">変換するString</param>
        /// <return>変換後　変換できなかった場合は0</return>
        /// </summary>
        public static int ConvInt(String str)
        {
            int num;
            if (Int32.TryParse(str, out num))
                return num;
            else
                return default(int);

        }

        /// <summary>
        /// String型 → Datetime型
        /// <param name="str">変換するString</param>
        /// <return>変換後　変換できなかった場合はデフォルト値</return>
        /// </summary>
        public static DateTime ConvDatetime(String str)
        {
            DateTime time;
            if (DateTime.TryParse(str, out time))
                return time;
            else
                return default(DateTime);
        }
        
        /// <summary>
        /// String型 → Datetime型（開始時間を含め）
        /// <param name="str">変換するString</param>
        /// <return>変換後　変換できなかった場合はデフォルト値</return>
        /// </summary>
        public static DateTime ConvStartDatetime(String str)
        {
            str += " 00:00:00";
            return ConvDatetime(str);
        }

        /// <summary>
        /// String型 → Datetime型（開始時間を含め）
        /// <param name="str">変換するString</param>
        /// <return>変換後　変換できなかった場合はデフォルト値</return>
        /// </summary>
        public static DateTime ConvEndDatetime(String str)
        {
            str += " 23:59:59";
            return ConvDatetime(str);
        }

        /// <summary>
        /// StringのEmpty判断
        /// </summary>
        /// <param name="str"></param>
        /// <returns>true:空欄、false:空欄ではない</returns>
        public static bool emptyString(String str)
        {
            return str == default(String) || str == "";

        }

        /// <summary>
        /// StringのEmpty判断
        /// </summary>
        /// <param name="str"></param>
        /// <returns>true:空欄、false:空欄ではない</returns>
        public static bool IsAlphanumeric(string input)
        {
            Regex regex = new Regex("^[a-zA-Z0-9]+$"); // 匹配只包含英文字母和数字的字符串
            return regex.IsMatch(input);
        }


        /// <summary>
        /// ログインしているユーザを取得する
        /// </summary>
        /// <returns>ユーザ</returns>
        public static UserModel getSessionUser()
        {
            if (Contains(System.Web.HttpContext.Current.Session, "User")) {
                UserModel user = (UserModel) System.Web.HttpContext.Current.Session["User"];
                return user;
            }
            else
            {
                return new UserModel();
            }

        }

        /// <summary>
        /// セッション値が存在するかどうかを示す値を取得します。
        /// 存在：true / ない：false。
        /// </summary>
        /// <param name="self">HttpSessionStateのインスタンス。</param>
        /// <param name="name">セッション値のキー名。</param>
        /// <returns>セッション値が存在する場合はtrue。しない場合はfalse。</returns>
        public static bool Contains(this HttpSessionState self, String name)
        {
            return self[name] != null;
        }


        /// <summary>
        /// 郵便番号の左側を取得
        /// </summary>
        /// <param name="str">郵便番号</param>
        /// <returns></returns>
        public static String GetZipCodeLeft(String str)
        {
            const int LEFT_LENGTH = 3;
            return LeftTruncate(FormatZipCode(str), LEFT_LENGTH);

        }

        /// <summary>
        /// 郵便番号の右側を取得
        /// </summary>
        /// <param name="str">郵便番号</param>
        /// <returns></returns>
        public static String GetZipCodeRight(String str)
        {
            const int RIGHT_LENGTH = 4;
            return RightTruncate(FormatZipCode(str), RIGHT_LENGTH);

        }

        /// <summary>
        /// 郵便番号形式
        /// </summary>
        /// <param name="str"></param>
        /// <returns></returns>
        private static String FormatZipCode(String str)
        {
            const int MAX_LENGTH = 7;
            if (str == null) return "";
            return LeftTruncate(Regex.Replace(str, @"[^0-9]", "").Replace(" ", ""), MAX_LENGTH);
        }

        /// <summary>
        /// Stringを切り捨てする（左から）
        /// </summary>
        /// <param name="value">値</param>
        /// <param name="length">文字数</param>
        /// <returns></returns>
        public static String LeftTruncate(String str, int length)
        {
            if (str == null) return null;
            if (str.Length > length) return str.Substring(0, length);
            return str;
        }

        /// <summary>
        /// Stringを切り捨てする（右から）
        /// </summary>
        /// <param name="value">値</param>
        /// <param name="length">文字数</param>
        /// <returns></returns>
        public static String RightTruncate(String str, int length)
        {
            if (str == null) return null;
            if (str.Length > length) return str.Substring(str.Length - length, length);
            return str;
        }

        
        /// <summary>
        /// ファイルの形式チェック（true：OK、false:NG）
        /// </summary>
        /// <param name="fileName">ファイル名</param>
        /// <param name="extension">ファイル形式 ※.は不要</param>
        /// <returns></returns>
        public static bool CheckExtension(String fileName ,String extension)
        {
            string fileExtension = Path.GetExtension(fileName);
            return (fileExtension == "." + extension);
        }

        /// <summary>
        /// 4M区分の表示ラベルを取得する
        /// </summary>
        /// <param name="changeType">4M区分</param>
        /// <returns>表示ラベル</returns>
        public static String GetChangeTypeLable(String changeType)
        {
            string label = "";
            if(!String.IsNullOrEmpty(changeType))
            {
                label = Const.CHANGE_TYPE_CODE_LABEL[changeType];
            }
            return label;
        }

        /// <summary>
        /// 対応状況の表示ラベルを取得する
        /// </summary>
        /// <param name="dealStatus">対応状況</param>
        /// <returns>表示ラベル</returns>
        public static String GetDealStatusLable(String dealStatus)
        {
            string label = "";
            if (!String.IsNullOrEmpty(dealStatus))
            {
                label = Const.DEAL_STATUS_CODE_LABEL[dealStatus];
            }
            return label;
        }

        /// <summary>
        /// 登録状態の表示ラベルを取得する
        /// </summary>
        /// <param name="dealStatus">登録状態</param>
        /// <returns>表示ラベル</returns>
        public static String GetRetisteStatusLable(String registeStatus)
        {
            string label = "";
            if (!String.IsNullOrEmpty(registeStatus))
            {
                label = Const.REGISTE_STATUS_CODE_LABEL[registeStatus];
            }
            return label;
        }

        /// <summary>
        /// 前後区分の表示ラベルを取得する
        /// </summary>
        /// <param name="dealStatus">前後区分</param>
        /// <returns>表示ラベル</returns>
        public static String GetBeforeAfterLable(String beforeAfter)
        {
            string label = "";
            if (!String.IsNullOrEmpty(beforeAfter))
            {
                label = Const.BEFORE_AFTER_CODE_LABEL[beforeAfter];
            }
            return label;
        }

        /// <summary>
        /// 管理Noを採番する
        /// </summary>
        /// <param name="lineCode">ラインコード</param>
        /// <returns>管理No</returns>
        public static String GetManageNo(String lineCode)
        {
            // ラインの組織コード取得
            LineRepository lineRepository = new LineRepository();
            m_line line = lineRepository.FindLineByCode(lineCode);
            String organizationCode = getSessionUser().OrganizationCode;
            if (line != null)
            {
                organizationCode = line.organization_code;
            }
            // 日付取得
            String dateStr = StaticUtil.GetToday().ToString("yyyyMMdd");
            // 採番データ取得
            NumberRepository repository = new NumberRepository();
            m_number numberData = repository.FindNumber(Const.NUMBER_TYPE_MANAGE_NO,
                organizationCode, dateStr);

            // 管理番号連番取得
            int seqNumber = 1;
            if (numberData == null)
            {
                numberData = new m_number();
                numberData.number_type = Const.NUMBER_TYPE_MANAGE_NO;
                numberData.number_key = organizationCode;
                numberData.number_unit = dateStr;
                numberData.seq_number = 1;
                repository.Add(numberData);
            } else
            {
                seqNumber = numberData.seq_number + 1;
                numberData.seq_number = seqNumber;
                repository.Update(numberData);
            }
            // 管理番号
            String manageNo = organizationCode + dateStr + "-" + Convert.ToString(seqNumber).PadLeft(2, '0');
            return manageNo;
        }

        /// <summary>
        /// 最終の対応状況
        /// </summary>
        /// <param name="modelState">モデルの状態</param>
        /// <returns></returns>
        public static String[] getReconfirmResult(string[][] ReconfirmResult)
        {
            int n = 0;
            string[] ReconfirmResult_status = new string[3];
            for (int i = ReconfirmResult.Length - 1; i >= 0; i--)
            {
                n = 0;
                for (int j = ReconfirmResult[i].Length - 1; j >= 1; j--)
                {
                    
                    if (!ReconfirmResult[i][0].IsEmpty())
                    {
                        
                        if (!ReconfirmResult[i][j].IsEmpty())
                        {
                            if (ReconfirmResult[i][j].Equals("9"))
                            {
                                ReconfirmResult_status[i] = "NG";
                                break;
                            }
                            else if (ReconfirmResult[i][j].Equals("1"))
                            {
                                ReconfirmResult_status[i] = "OK";
                                break;

                            }
                        }
                        else
                        {
                            n++;
                        }
                    }
                }
                if (n == 6)
                {
                    ReconfirmResult_status[i] = "NO";
                }
            }
            return ReconfirmResult_status;

        }

        /// <summary>
        /// 教育ステータス変更
        /// </summary>
        /// <param name="modelState">モデルの状態</param>
        /// <returns></returns>
        public static bool TrainingCheckStatus(TrainingResultViewModel viewModel)
        {
            bool ChangeStatus = true;
            if (!string.IsNullOrEmpty(viewModel.EfuNo1) && string.IsNullOrEmpty(viewModel.ProductNumber1)) {
                ChangeStatus = false;
            } else if (!string.IsNullOrEmpty(viewModel.EfuNo2) && string.IsNullOrEmpty(viewModel.ProductNumber2))
            {
                ChangeStatus = false;
            } else if (!string.IsNullOrEmpty(viewModel.EfuNo3) && string.IsNullOrEmpty(viewModel.ProductNumber3)) {
                ChangeStatus = false;
            }
            return ChangeStatus;

        }


        /// <summary>
        /// 検査行数
        /// </summary>
        /// <param name="modelState">モデルの状態</param>
        /// <returns></returns>
        public static string CheckRow(string[] ReconfirmResult)
        {
            int row = 3;
            foreach (string item in ReconfirmResult) {
                if (!string.IsNullOrEmpty(item)) {
                    row++;
                }
            }
            return Convert.ToString(row);

        }

        /// <summary>
        /// 検査確認ステータス変更
        /// </summary>
        /// <param name="modelState">モデルの状態</param>
        /// <returns></returns>
        public static bool CheckStatus(CheckResultViewModel viewModel)
        {
            bool ChangeStatus = true;
            if (!string.IsNullOrEmpty(viewModel.EfuNo1) && string.IsNullOrEmpty(viewModel.ProductNumber1))
            {
                ChangeStatus = false;
            }
            else if (!string.IsNullOrEmpty(viewModel.EfuNo2) && string.IsNullOrEmpty(viewModel.ProductNumber2))
            {
                ChangeStatus = false;
            }
            else if (!string.IsNullOrEmpty(viewModel.EfuNo3) && string.IsNullOrEmpty(viewModel.ProductNumber3))
            {
                ChangeStatus = false;
            }
            return ChangeStatus;

        }

        #region UserWebGridの取得（検索）
        /// <summary>
        /// UserWebGridの取得
        /// <param name="userName"> 担当者名 </param>
        /// </summary>
        public static WebGrid GetUserGrid(String userName)
        {
            WorkerRepository repository = new WorkerRepository();
            IEnumerable<UserModel> Worklist = repository.FindWorkerBySearch(userName);

            WebGrid gridD = new WebGrid(source: Worklist, rowsPerPage: 8, canSort: true, ajaxUpdateContainerId: "gridD");

            return gridD;
        }
        #endregion

        #region Userのカラムの取得
        /// <summary>
        /// Deviceのカラム
        /// <param name="grid"> WebGrid </param>
        /// </summary>
        public static WebGridColumn[] GetUserGridColumn(WebGrid gridD)
        {
            WebGridColumn[] gridColumns = new WebGridColumn[]
            {
                gridD.Column(columnName: "UserId", header: "社員コード", style:"webgrid-cd")
                ,gridD.Column(columnName: "UserNm", header: "担当者名", style:"webgrid-data")
            };

            return gridColumns;
        }
        #endregion

        #region 下位組織含まるアクセス組織リスト取得
        /// <summary>
        /// 下位組織含まるアクセス組織リスト取得
        /// <returns>アクセス組織リスト</returns>
        /// </summary>
        public static List<String> GetOrganizationList()
        {
            // ログインユーザの組織コード取得
            UserModel loginUser = getSessionUser();
            String organizationCode = loginUser.OrganizationCode;
            List<String> organizationCodeList = new List<String>();
            organizationCodeList.Add(organizationCode);

            // 下位組織を全て取得する
            OrganizationRepository repository = new OrganizationRepository();
            List<String> parentCodeList = new List<String>();
            parentCodeList.Add(organizationCode);
            while (true)
            {
                List<m_organization> subOrganizationList = repository.FindSubOrganizationList(parentCodeList);
                if(subOrganizationList == null || subOrganizationList.Count == 0)
                {
                    break;
                }
                parentCodeList.Clear();
                foreach (m_organization organization in subOrganizationList)
                {
                    parentCodeList.Add(organization.organization_code);
                    organizationCodeList.Add(organization.organization_code);
                }
            }

            return organizationCodeList;
        }
        #endregion

        #region 戻るパス取得
        /// <summary>
        /// 戻るパスの取得
        /// <param name="SourcePath"> 遷移元パス </param>
        /// <param name="defaultPath"> デフォルトパス </param>
        /// </summary>
        public static PathBean GetReturnPath(String sourcePath, String defaultPath)
        {
            PathBean pathData = new PathBean();
            pathData.ReturnPath = defaultPath;
            // 遷移元を取得
            if (!String.IsNullOrEmpty(sourcePath))
            {
                String[] path = sourcePath.Split(Const.SPLIT_COMMA_CHAR);
                // 戻るパス設定
                pathData.ReturnPath = path[path.Length - 1];
                // 遷移元パス保持
                List<String> pathList = new List<String>();
                for(int index = 0; index < path.Length - 1; index++) {
                    pathList.Add(path[index]);
                }
                if(pathList.Count > 0)
                {
                    pathData.SourcePath = String.Join(Const.SPLIT_COMMA_STRING, pathList);
                }
            }
            return pathData;
        }
        #endregion

        #region 遷移元パス取得
        /// <summary>
        /// 遷移先に渡すパスの取得
        /// <param name="SourcePath"> 遷移元パス </param>
        /// <param name="thisPath"> 現在画面パス </param>
        /// </summary>
        public static String GetSourcePath(String sourcePath, String thisPath)
        {
            // 遷移元パス保持
            List<String> pathList = new List<String>();
            // 遷移元を取得
            if (!String.IsNullOrEmpty(sourcePath))
            {
                String[] path = sourcePath.Split(Const.SPLIT_COMMA_CHAR);
                
                for (int index = 0; index < path.Length; index++)
                {
                    pathList.Add(path[index]);
                }
            }
            pathList.Add(thisPath);
            return String.Join(Const.SPLIT_COMMA_STRING, pathList);
        }
        #endregion
    }
}